'''
Created on 2023-02-19

@author: wf
'''
import meta

class Version(object):
    '''
    Version handling for pyMetaModel
    '''
    name="pyMetaModel"
    description='pyMetaModel: MetaModel for Knowledge Graphs'
    version=meta.__version__
    date = '2022-11-30'
    updated = '2023-02-19'
    authors='Wolfgang Fahl'
    doc_url="https://wiki.bitplan.com/index.php/pyMetaModel"
    chat_url="https://github.com/WolfgangFahl/pyMetaModel/discussions"
    cm_url="https://github.com/WolfgangFahl/pyMetaModel"
    license=f'''Copyright 2022 contributors. All rights reserved.
  Licensed under the Apache License 2.0
  http://www.apache.org/licenses/LICENSE-2.0
  Distributed on an "AS IS" basis without warranties
  or conditions of any kind, either express or implied.'''
    longDescription=f"""{name} version {version}
{description}
  Created by {authors} on {date} last updated {updated}"""

        


# Auto generated from TeachingSchema.yaml by pythongen.py version: 0.9.0
# Generation date: 2023-03-08T09:54:50
# Schema: TeachingSchema
#
# id: TeachingSchema
# description:
# license:

import dataclasses
import sys
import re
from jsonasobj2 import JsonObj, as_dict
from typing import Optional, List, Union, Dict, ClassVar, Any
from dataclasses import dataclass
from linkml_runtime.linkml_model.meta import EnumDefinition, PermissibleValue, PvFormulaOptions

from linkml_runtime.utils.slot import Slot
from linkml_runtime.utils.metamodelcore import empty_list, empty_dict, bnode
from linkml_runtime.utils.yamlutils import YAMLRoot, extended_str, extended_float, extended_int
from linkml_runtime.utils.dataclass_extensions_376 import dataclasses_init_fn_with_kwargs
from linkml_runtime.utils.formatutils import camelcase, underscore, sfx
from linkml_runtime.utils.enumerations import EnumDefinitionImpl
from rdflib import Namespace, URIRef
from linkml_runtime.utils.curienamespace import CurieNamespace
from linkml_runtime.utils.metamodelcore import Bool, Curie, Decimal, ElementIdentifier, NCName, NodeIdentifier, URI, URIorCURIE, XSDDate, XSDDateTime, XSDTime

metamodel_version = "1.7.0"
version = None

# Overwrite dataclasses _init_fn to add **kwargs in __init__
dataclasses._init_fn = dataclasses_init_fn_with_kwargs

# Namespaces
TEACHINGSCHEMA = CurieNamespace('TeachingSchema', 'TeachingSchema/')
LINKML = CurieNamespace('linkml', 'https://w3id.org/linkml/')
SCHEMA = CurieNamespace('schema', 'http://schema.org/')
SHEX = CurieNamespace('shex', 'http://www.w3.org/ns/shex#')
XSD = CurieNamespace('xsd', 'http://www.w3.org/2001/XMLSchema#')
DEFAULT_ = TEACHINGSCHEMA


# Types
class String(str):
    """ A character string """
    type_class_uri = XSD.string
    type_class_curie = "xsd:string"
    type_name = "string"
    type_model_uri = TEACHINGSCHEMA.String


class Integer(int):
    """ An integer """
    type_class_uri = XSD.integer
    type_class_curie = "xsd:integer"
    type_name = "integer"
    type_model_uri = TEACHINGSCHEMA.Integer


class Boolean(Bool):
    """ A binary (true or false) value """
    type_class_uri = XSD.boolean
    type_class_curie = "xsd:boolean"
    type_name = "boolean"
    type_model_uri = TEACHINGSCHEMA.Boolean


class Float(float):
    """ A real number that conforms to the xsd:float specification """
    type_class_uri = XSD.float
    type_class_curie = "xsd:float"
    type_name = "float"
    type_model_uri = TEACHINGSCHEMA.Float


class Double(float):
    """ A real number that conforms to the xsd:double specification """
    type_class_uri = XSD.double
    type_class_curie = "xsd:double"
    type_name = "double"
    type_model_uri = TEACHINGSCHEMA.Double


class Decimal(Decimal):
    """ A real number with arbitrary precision that conforms to the xsd:decimal specification """
    type_class_uri = XSD.decimal
    type_class_curie = "xsd:decimal"
    type_name = "decimal"
    type_model_uri = TEACHINGSCHEMA.Decimal


class Time(XSDTime):
    """ A time object represents a (local) time of day, independent of any particular day """
    type_class_uri = XSD.dateTime
    type_class_curie = "xsd:dateTime"
    type_name = "time"
    type_model_uri = TEACHINGSCHEMA.Time


class Date(XSDDate):
    """ a date (year, month and day) in an idealized calendar """
    type_class_uri = XSD.date
    type_class_curie = "xsd:date"
    type_name = "date"
    type_model_uri = TEACHINGSCHEMA.Date


class Datetime(XSDDateTime):
    """ The combination of a date and time """
    type_class_uri = XSD.dateTime
    type_class_curie = "xsd:dateTime"
    type_name = "datetime"
    type_model_uri = TEACHINGSCHEMA.Datetime


class DateOrDatetime(str):
    """ Either a date or a datetime """
    type_class_uri = LINKML.DateOrDatetime
    type_class_curie = "linkml:DateOrDatetime"
    type_name = "date_or_datetime"
    type_model_uri = TEACHINGSCHEMA.DateOrDatetime


class Uriorcurie(URIorCURIE):
    """ a URI or a CURIE """
    type_class_uri = XSD.anyURI
    type_class_curie = "xsd:anyURI"
    type_name = "uriorcurie"
    type_model_uri = TEACHINGSCHEMA.Uriorcurie


class Curie(Curie):
    """ a compact URI """
    type_class_uri = XSD.string
    type_class_curie = "xsd:string"
    type_name = "curie"
    type_model_uri = TEACHINGSCHEMA.Curie


class Uri(URI):
    """ a complete URI """
    type_class_uri = XSD.anyURI
    type_class_curie = "xsd:anyURI"
    type_name = "uri"
    type_model_uri = TEACHINGSCHEMA.Uri


class Ncname(NCName):
    """ Prefix part of CURIE """
    type_class_uri = XSD.string
    type_class_curie = "xsd:string"
    type_name = "ncname"
    type_model_uri = TEACHINGSCHEMA.Ncname


class Objectidentifier(ElementIdentifier):
    """ A URI or CURIE that represents an object in the model. """
    type_class_uri = SHEX.iri
    type_class_curie = "shex:iri"
    type_name = "objectidentifier"
    type_model_uri = TEACHINGSCHEMA.Objectidentifier


class Nodeidentifier(NodeIdentifier):
    """ A URI, CURIE or BNODE that represents a node in a model. """
    type_class_uri = SHEX.nonLiteral
    type_class_curie = "shex:nonLiteral"
    type_name = "nodeidentifier"
    type_model_uri = TEACHINGSCHEMA.Nodeidentifier


# Class references



@dataclass
class Chapter(YAMLRoot):
    """
    a chapter of a lecture
    """
    _inherited_slots: ClassVar[List[str]] = []

    class_class_uri: ClassVar[URIRef] = TEACHINGSCHEMA.Chapter
    class_class_curie: ClassVar[str] = "TeachingSchema:Chapter"
    class_name: ClassVar[str] = "Chapter"
    class_model_uri: ClassVar[URIRef] = TEACHINGSCHEMA.Chapter

    key: Optional[str] = None
    no: Optional[str] = None
    pres_id: Optional[str] = None
    week: Optional[str] = None
    de: Optional[str] = None
    en: Optional[str] = None

    def __post_init__(self, *_: List[str], **kwargs: Dict[str, Any]):
        if self.key is not None and not isinstance(self.key, str):
            self.key = str(self.key)

        if self.no is not None and not isinstance(self.no, str):
            self.no = str(self.no)

        if self.pres_id is not None and not isinstance(self.pres_id, str):
            self.pres_id = str(self.pres_id)

        if self.week is not None and not isinstance(self.week, str):
            self.week = str(self.week)

        if self.de is not None and not isinstance(self.de, str):
            self.de = str(self.de)

        if self.en is not None and not isinstance(self.en, str):
            self.en = str(self.en)

        super().__post_init__(**kwargs)


@dataclass
class Presentation(YAMLRoot):
    """
    a presentation of a lecture
    """
    _inherited_slots: ClassVar[List[str]] = []

    class_class_uri: ClassVar[URIRef] = TEACHINGSCHEMA.Presentation
    class_class_curie: ClassVar[str] = "TeachingSchema:Presentation"
    class_name: ClassVar[str] = "Presentation"
    class_model_uri: ClassVar[URIRef] = TEACHINGSCHEMA.Presentation

    key: Optional[str] = None
    num: Optional[str] = None
    name: Optional[str] = None
    ppt_file: Optional[str] = None
    sciebo: Optional[str] = None
    gitlab: Optional[str] = None
    moodle: Optional[str] = None

    def __post_init__(self, *_: List[str], **kwargs: Dict[str, Any]):
        if self.key is not None and not isinstance(self.key, str):
            self.key = str(self.key)

        if self.num is not None and not isinstance(self.num, str):
            self.num = str(self.num)

        if self.name is not None and not isinstance(self.name, str):
            self.name = str(self.name)

        if self.ppt_file is not None and not isinstance(self.ppt_file, str):
            self.ppt_file = str(self.ppt_file)

        if self.sciebo is not None and not isinstance(self.sciebo, str):
            self.sciebo = str(self.sciebo)

        if self.gitlab is not None and not isinstance(self.gitlab, str):
            self.gitlab = str(self.gitlab)

        if self.moodle is not None and not isinstance(self.moodle, str):
            self.moodle = str(self.moodle)

        super().__post_init__(**kwargs)


@dataclass
class LearningGoal(YAMLRoot):
    """
    a LearningGoal of a lecture
    """
    _inherited_slots: ClassVar[List[str]] = []

    class_class_uri: ClassVar[URIRef] = TEACHINGSCHEMA.LearningGoal
    class_class_curie: ClassVar[str] = "TeachingSchema:LearningGoal"
    class_name: ClassVar[str] = "LearningGoal"
    class_model_uri: ClassVar[URIRef] = TEACHINGSCHEMA.LearningGoal

    key: Optional[str] = None
    qKey: Optional[str] = None
    id: Optional[str] = None
    chapter: Optional[str] = None
    chapterKey: Optional[str] = None
    subChapter: Optional[str] = None
    subChapterKey: Optional[str] = None
    relevance: Optional[str] = None
    keywords: Optional[str] = None
    since: Optional[Union[str, XSDDate]] = None
    until: Optional[Union[str, XSDDate]] = None

    def __post_init__(self, *_: List[str], **kwargs: Dict[str, Any]):
        if self.key is not None and not isinstance(self.key, str):
            self.key = str(self.key)

        if self.qKey is not None and not isinstance(self.qKey, str):
            self.qKey = str(self.qKey)

        if self.id is not None and not isinstance(self.id, str):
            self.id = str(self.id)

        if self.chapter is not None and not isinstance(self.chapter, str):
            self.chapter = str(self.chapter)

        if self.chapterKey is not None and not isinstance(self.chapterKey, str):
            self.chapterKey = str(self.chapterKey)

        if self.subChapter is not None and not isinstance(self.subChapter, str):
            self.subChapter = str(self.subChapter)

        if self.subChapterKey is not None and not isinstance(self.subChapterKey, str):
            self.subChapterKey = str(self.subChapterKey)

        if self.relevance is not None and not isinstance(self.relevance, str):
            self.relevance = str(self.relevance)

        if self.keywords is not None and not isinstance(self.keywords, str):
            self.keywords = str(self.keywords)

        if self.since is not None and not isinstance(self.since, XSDDate):
            self.since = XSDDate(self.since)

        if self.until is not None and not isinstance(self.until, XSDDate):
            self.until = XSDDate(self.until)

        super().__post_init__(**kwargs)


@dataclass
class Keyword(YAMLRoot):
    """
    a keyword
    """
    _inherited_slots: ClassVar[List[str]] = []

    class_class_uri: ClassVar[URIRef] = TEACHINGSCHEMA.Keyword
    class_class_curie: ClassVar[str] = "TeachingSchema:Keyword"
    class_name: ClassVar[str] = "Keyword"
    class_model_uri: ClassVar[URIRef] = TEACHINGSCHEMA.Keyword

    key: Optional[str] = None
    aliases: Optional[str] = None
    de: Optional[str] = None
    en: Optional[str] = None
    wikidataid: Optional[str] = None
    links: Optional[str] = None
    literature: Optional[str] = None
    urls: Optional[str] = None
    stackoverflow_tag: Optional[str] = None

    def __post_init__(self, *_: List[str], **kwargs: Dict[str, Any]):
        if self.key is not None and not isinstance(self.key, str):
            self.key = str(self.key)

        if self.aliases is not None and not isinstance(self.aliases, str):
            self.aliases = str(self.aliases)

        if self.de is not None and not isinstance(self.de, str):
            self.de = str(self.de)

        if self.en is not None and not isinstance(self.en, str):
            self.en = str(self.en)

        if self.wikidataid is not None and not isinstance(self.wikidataid, str):
            self.wikidataid = str(self.wikidataid)

        if self.links is not None and not isinstance(self.links, str):
            self.links = str(self.links)

        if self.literature is not None and not isinstance(self.literature, str):
            self.literature = str(self.literature)

        if self.urls is not None and not isinstance(self.urls, str):
            self.urls = str(self.urls)

        if self.stackoverflow_tag is not None and not isinstance(self.stackoverflow_tag, str):
            self.stackoverflow_tag = str(self.stackoverflow_tag)

        super().__post_init__(**kwargs)


@dataclass
class Slide(YAMLRoot):
    """
    I am a presentation slide
    """
    _inherited_slots: ClassVar[List[str]] = []

    class_class_uri: ClassVar[URIRef] = TEACHINGSCHEMA.Slide
    class_class_curie: ClassVar[str] = "TeachingSchema:Slide"
    class_name: ClassVar[str] = "Slide"
    class_model_uri: ClassVar[URIRef] = TEACHINGSCHEMA.Slide

    qkey: Optional[str] = None
    pkey: Optional[str] = None
    basename: Optional[str] = None
    page: Optional[str] = None
    name: Optional[str] = None
    title: Optional[str] = None
    keywords: Optional[str] = None
    links: Optional[str] = None
    literature: Optional[str] = None
    learningGoal: Optional[str] = None

    def __post_init__(self, *_: List[str], **kwargs: Dict[str, Any]):
        if self.qkey is not None and not isinstance(self.qkey, str):
            self.qkey = str(self.qkey)

        if self.pkey is not None and not isinstance(self.pkey, str):
            self.pkey = str(self.pkey)

        if self.basename is not None and not isinstance(self.basename, str):
            self.basename = str(self.basename)

        if self.page is not None and not isinstance(self.page, str):
            self.page = str(self.page)

        if self.name is not None and not isinstance(self.name, str):
            self.name = str(self.name)

        if self.title is not None and not isinstance(self.title, str):
            self.title = str(self.title)

        if self.keywords is not None and not isinstance(self.keywords, str):
            self.keywords = str(self.keywords)

        if self.links is not None and not isinstance(self.links, str):
            self.links = str(self.links)

        if self.literature is not None and not isinstance(self.literature, str):
            self.literature = str(self.literature)

        if self.learningGoal is not None and not isinstance(self.learningGoal, str):
            self.learningGoal = str(self.learningGoal)

        super().__post_init__(**kwargs)


@dataclass
class Check(YAMLRoot):
    """
    I am a quality check
    """
    _inherited_slots: ClassVar[List[str]] = []

    class_class_uri: ClassVar[URIRef] = TEACHINGSCHEMA.Check
    class_class_curie: ClassVar[str] = "TeachingSchema:Check"
    class_name: ClassVar[str] = "Check"
    class_model_uri: ClassVar[URIRef] = TEACHINGSCHEMA.Check

    qKey: Optional[str] = None
    key: Optional[str] = None
    concept: Optional[str] = None
    since: Optional[Union[str, XSDDate]] = None
    check: Optional[str] = None
    problem: Optional[str] = None
    ok: Optional[float] = None
    total: Optional[float] = None
    pain: Optional[float] = None

    def __post_init__(self, *_: List[str], **kwargs: Dict[str, Any]):
        if self.qKey is not None and not isinstance(self.qKey, str):
            self.qKey = str(self.qKey)

        if self.key is not None and not isinstance(self.key, str):
            self.key = str(self.key)

        if self.concept is not None and not isinstance(self.concept, str):
            self.concept = str(self.concept)

        if self.since is not None and not isinstance(self.since, XSDDate):
            self.since = XSDDate(self.since)

        if self.check is not None and not isinstance(self.check, str):
            self.check = str(self.check)

        if self.problem is not None and not isinstance(self.problem, str):
            self.problem = str(self.problem)

        if self.ok is not None and not isinstance(self.ok, float):
            self.ok = float(self.ok)

        if self.total is not None and not isinstance(self.total, float):
            self.total = float(self.total)

        if self.pain is not None and not isinstance(self.pain, float):
            self.pain = float(self.pain)

        super().__post_init__(**kwargs)


@dataclass
class Issue(YAMLRoot):
    """
    I am a an issue
    """
    _inherited_slots: ClassVar[List[str]] = []

    class_class_uri: ClassVar[URIRef] = TEACHINGSCHEMA.Issue
    class_class_curie: ClassVar[str] = "TeachingSchema:Issue"
    class_name: ClassVar[str] = "Issue"
    class_model_uri: ClassVar[URIRef] = TEACHINGSCHEMA.Issue

    item: Optional[str] = None
    check: Optional[str] = None
    fix: Optional[str] = None

    def __post_init__(self, *_: List[str], **kwargs: Dict[str, Any]):
        if self.item is not None and not isinstance(self.item, str):
            self.item = str(self.item)

        if self.check is not None and not isinstance(self.check, str):
            self.check = str(self.check)

        if self.fix is not None and not isinstance(self.fix, str):
            self.fix = str(self.fix)

        super().__post_init__(**kwargs)


@dataclass
class RelevanceLevel(YAMLRoot):
    """
    I am a relevance level
    """
    _inherited_slots: ClassVar[List[str]] = []

    class_class_uri: ClassVar[URIRef] = TEACHINGSCHEMA.RelevanceLevel
    class_class_curie: ClassVar[str] = "TeachingSchema:RelevanceLevel"
    class_name: ClassVar[str] = "RelevanceLevel"
    class_model_uri: ClassVar[URIRef] = TEACHINGSCHEMA.RelevanceLevel

    id: Optional[str] = None
    expectation: Optional[str] = None
    examination: Optional[str] = None
    description: Optional[str] = None

    def __post_init__(self, *_: List[str], **kwargs: Dict[str, Any]):
        if self.id is not None and not isinstance(self.id, str):
            self.id = str(self.id)

        if self.expectation is not None and not isinstance(self.expectation, str):
            self.expectation = str(self.expectation)

        if self.examination is not None and not isinstance(self.examination, str):
            self.examination = str(self.examination)

        if self.description is not None and not isinstance(self.description, str):
            self.description = str(self.description)

        super().__post_init__(**kwargs)


@dataclass
class Publication(YAMLRoot):
    """
    A Publication is e.g. a scholarly article
    """
    _inherited_slots: ClassVar[List[str]] = []

    class_class_uri: ClassVar[URIRef] = TEACHINGSCHEMA.Publication
    class_class_curie: ClassVar[str] = "TeachingSchema:Publication"
    class_name: ClassVar[str] = "Publication"
    class_model_uri: ClassVar[URIRef] = TEACHINGSCHEMA.Publication

    id: Optional[str] = None
    doi: Optional[str] = None
    wikidataid: Optional[str] = None
    isbn: Optional[str] = None
    title: Optional[str] = None
    authors: Optional[str] = None
    publisher: Optional[str] = None
    year: Optional[float] = None
    pdfUrl: Optional[Union[str, URI]] = None

    def __post_init__(self, *_: List[str], **kwargs: Dict[str, Any]):
        if self.id is not None and not isinstance(self.id, str):
            self.id = str(self.id)

        if self.doi is not None and not isinstance(self.doi, str):
            self.doi = str(self.doi)

        if self.wikidataid is not None and not isinstance(self.wikidataid, str):
            self.wikidataid = str(self.wikidataid)

        if self.isbn is not None and not isinstance(self.isbn, str):
            self.isbn = str(self.isbn)

        if self.title is not None and not isinstance(self.title, str):
            self.title = str(self.title)

        if self.authors is not None and not isinstance(self.authors, str):
            self.authors = str(self.authors)

        if self.publisher is not None and not isinstance(self.publisher, str):
            self.publisher = str(self.publisher)

        if self.year is not None and not isinstance(self.year, float):
            self.year = float(self.year)

        if self.pdfUrl is not None and not isinstance(self.pdfUrl, URI):
            self.pdfUrl = URI(self.pdfUrl)

        super().__post_init__(**kwargs)


@dataclass
class Sheet(YAMLRoot):
    """
    I am a sheet in a spreadsheet
    """
    _inherited_slots: ClassVar[List[str]] = []

    class_class_uri: ClassVar[URIRef] = TEACHINGSCHEMA.Sheet
    class_class_curie: ClassVar[str] = "TeachingSchema:Sheet"
    class_name: ClassVar[str] = "Sheet"
    class_model_uri: ClassVar[URIRef] = TEACHINGSCHEMA.Sheet

    gid: Optional[str] = None
    concept: Optional[str] = None
    url: Optional[str] = None

    def __post_init__(self, *_: List[str], **kwargs: Dict[str, Any]):
        if self.gid is not None and not isinstance(self.gid, str):
            self.gid = str(self.gid)

        if self.concept is not None and not isinstance(self.concept, str):
            self.concept = str(self.concept)

        if self.url is not None and not isinstance(self.url, str):
            self.url = str(self.url)

        super().__post_init__(**kwargs)


# Enumerations


# Slots
class slots:
    pass

slots.key = Slot(uri=TEACHINGSCHEMA.key, name="key", curie=TEACHINGSCHEMA.curie('key'),
                   model_uri=TEACHINGSCHEMA.key, domain=None, range=Optional[str])

slots.no = Slot(uri=TEACHINGSCHEMA.no, name="no", curie=TEACHINGSCHEMA.curie('no'),
                   model_uri=TEACHINGSCHEMA.no, domain=None, range=Optional[str])

slots.pres_id = Slot(uri=TEACHINGSCHEMA.pres_id, name="pres_id", curie=TEACHINGSCHEMA.curie('pres_id'),
                   model_uri=TEACHINGSCHEMA.pres_id, domain=None, range=Optional[str])

slots.week = Slot(uri=TEACHINGSCHEMA.week, name="week", curie=TEACHINGSCHEMA.curie('week'),
                   model_uri=TEACHINGSCHEMA.week, domain=None, range=Optional[str])

slots.de = Slot(uri=TEACHINGSCHEMA.de, name="de", curie=TEACHINGSCHEMA.curie('de'),
                   model_uri=TEACHINGSCHEMA.de, domain=None, range=Optional[str])

slots.en = Slot(uri=TEACHINGSCHEMA.en, name="en", curie=TEACHINGSCHEMA.curie('en'),
                   model_uri=TEACHINGSCHEMA.en, domain=None, range=Optional[str])

slots.num = Slot(uri=TEACHINGSCHEMA.num, name="num", curie=TEACHINGSCHEMA.curie('num'),
                   model_uri=TEACHINGSCHEMA.num, domain=None, range=Optional[str])

slots.name = Slot(uri=TEACHINGSCHEMA.name, name="name", curie=TEACHINGSCHEMA.curie('name'),
                   model_uri=TEACHINGSCHEMA.name, domain=None, range=Optional[str])

slots.ppt_file = Slot(uri=TEACHINGSCHEMA.ppt_file, name="ppt_file", curie=TEACHINGSCHEMA.curie('ppt_file'),
                   model_uri=TEACHINGSCHEMA.ppt_file, domain=None, range=Optional[str])

slots.sciebo = Slot(uri=TEACHINGSCHEMA.sciebo, name="sciebo", curie=TEACHINGSCHEMA.curie('sciebo'),
                   model_uri=TEACHINGSCHEMA.sciebo, domain=None, range=Optional[str])

slots.gitlab = Slot(uri=TEACHINGSCHEMA.gitlab, name="gitlab", curie=TEACHINGSCHEMA.curie('gitlab'),
                   model_uri=TEACHINGSCHEMA.gitlab, domain=None, range=Optional[str])

slots.moodle = Slot(uri=TEACHINGSCHEMA.moodle, name="moodle", curie=TEACHINGSCHEMA.curie('moodle'),
                   model_uri=TEACHINGSCHEMA.moodle, domain=None, range=Optional[str])

slots.qKey = Slot(uri=TEACHINGSCHEMA.qKey, name="qKey", curie=TEACHINGSCHEMA.curie('qKey'),
                   model_uri=TEACHINGSCHEMA.qKey, domain=None, range=Optional[str])

slots.id = Slot(uri=TEACHINGSCHEMA.id, name="id", curie=TEACHINGSCHEMA.curie('id'),
                   model_uri=TEACHINGSCHEMA.id, domain=None, range=Optional[str])

slots.chapter = Slot(uri=TEACHINGSCHEMA.chapter, name="chapter", curie=TEACHINGSCHEMA.curie('chapter'),
                   model_uri=TEACHINGSCHEMA.chapter, domain=None, range=Optional[str])

slots.chapterKey = Slot(uri=TEACHINGSCHEMA.chapterKey, name="chapterKey", curie=TEACHINGSCHEMA.curie('chapterKey'),
                   model_uri=TEACHINGSCHEMA.chapterKey, domain=None, range=Optional[str])

slots.subChapter = Slot(uri=TEACHINGSCHEMA.subChapter, name="subChapter", curie=TEACHINGSCHEMA.curie('subChapter'),
                   model_uri=TEACHINGSCHEMA.subChapter, domain=None, range=Optional[str])

slots.subChapterKey = Slot(uri=TEACHINGSCHEMA.subChapterKey, name="subChapterKey", curie=TEACHINGSCHEMA.curie('subChapterKey'),
                   model_uri=TEACHINGSCHEMA.subChapterKey, domain=None, range=Optional[str])

slots.relevance = Slot(uri=TEACHINGSCHEMA.relevance, name="relevance", curie=TEACHINGSCHEMA.curie('relevance'),
                   model_uri=TEACHINGSCHEMA.relevance, domain=None, range=Optional[str])

slots.keywords = Slot(uri=TEACHINGSCHEMA.keywords, name="keywords", curie=TEACHINGSCHEMA.curie('keywords'),
                   model_uri=TEACHINGSCHEMA.keywords, domain=None, range=Optional[str])

slots.since = Slot(uri=TEACHINGSCHEMA.since, name="since", curie=TEACHINGSCHEMA.curie('since'),
                   model_uri=TEACHINGSCHEMA.since, domain=None, range=Optional[Union[str, XSDDate]])

slots.until = Slot(uri=TEACHINGSCHEMA.until, name="until", curie=TEACHINGSCHEMA.curie('until'),
                   model_uri=TEACHINGSCHEMA.until, domain=None, range=Optional[Union[str, XSDDate]])

slots.aliases = Slot(uri=TEACHINGSCHEMA.aliases, name="aliases", curie=TEACHINGSCHEMA.curie('aliases'),
                   model_uri=TEACHINGSCHEMA.aliases, domain=None, range=Optional[str])

slots.wikidataid = Slot(uri=TEACHINGSCHEMA.wikidataid, name="wikidataid", curie=TEACHINGSCHEMA.curie('wikidataid'),
                   model_uri=TEACHINGSCHEMA.wikidataid, domain=None, range=Optional[str])

slots.links = Slot(uri=TEACHINGSCHEMA.links, name="links", curie=TEACHINGSCHEMA.curie('links'),
                   model_uri=TEACHINGSCHEMA.links, domain=None, range=Optional[str])

slots.literature = Slot(uri=TEACHINGSCHEMA.literature, name="literature", curie=TEACHINGSCHEMA.curie('literature'),
                   model_uri=TEACHINGSCHEMA.literature, domain=None, range=Optional[str])

slots.urls = Slot(uri=TEACHINGSCHEMA.urls, name="urls", curie=TEACHINGSCHEMA.curie('urls'),
                   model_uri=TEACHINGSCHEMA.urls, domain=None, range=Optional[str])

slots.stackoverflow_tag = Slot(uri=TEACHINGSCHEMA.stackoverflow_tag, name="stackoverflow_tag", curie=TEACHINGSCHEMA.curie('stackoverflow_tag'),
                   model_uri=TEACHINGSCHEMA.stackoverflow_tag, domain=None, range=Optional[str])

slots.qkey = Slot(uri=TEACHINGSCHEMA.qkey, name="qkey", curie=TEACHINGSCHEMA.curie('qkey'),
                   model_uri=TEACHINGSCHEMA.qkey, domain=None, range=Optional[str])

slots.pkey = Slot(uri=TEACHINGSCHEMA.pkey, name="pkey", curie=TEACHINGSCHEMA.curie('pkey'),
                   model_uri=TEACHINGSCHEMA.pkey, domain=None, range=Optional[str])

slots.basename = Slot(uri=TEACHINGSCHEMA.basename, name="basename", curie=TEACHINGSCHEMA.curie('basename'),
                   model_uri=TEACHINGSCHEMA.basename, domain=None, range=Optional[str])

slots.page = Slot(uri=TEACHINGSCHEMA.page, name="page", curie=TEACHINGSCHEMA.curie('page'),
                   model_uri=TEACHINGSCHEMA.page, domain=None, range=Optional[str])

slots.title = Slot(uri=TEACHINGSCHEMA.title, name="title", curie=TEACHINGSCHEMA.curie('title'),
                   model_uri=TEACHINGSCHEMA.title, domain=None, range=Optional[str])

slots.learningGoal = Slot(uri=TEACHINGSCHEMA.learningGoal, name="learningGoal", curie=TEACHINGSCHEMA.curie('learningGoal'),
                   model_uri=TEACHINGSCHEMA.learningGoal, domain=None, range=Optional[str])

slots.concept = Slot(uri=TEACHINGSCHEMA.concept, name="concept", curie=TEACHINGSCHEMA.curie('concept'),
                   model_uri=TEACHINGSCHEMA.concept, domain=None, range=Optional[str])

slots.check = Slot(uri=TEACHINGSCHEMA.check, name="check", curie=TEACHINGSCHEMA.curie('check'),
                   model_uri=TEACHINGSCHEMA.check, domain=None, range=Optional[str])

slots.problem = Slot(uri=TEACHINGSCHEMA.problem, name="problem", curie=TEACHINGSCHEMA.curie('problem'),
                   model_uri=TEACHINGSCHEMA.problem, domain=None, range=Optional[str])

slots.ok = Slot(uri=TEACHINGSCHEMA.ok, name="ok", curie=TEACHINGSCHEMA.curie('ok'),
                   model_uri=TEACHINGSCHEMA.ok, domain=None, range=Optional[float])

slots.total = Slot(uri=TEACHINGSCHEMA.total, name="total", curie=TEACHINGSCHEMA.curie('total'),
                   model_uri=TEACHINGSCHEMA.total, domain=None, range=Optional[float])

slots.pain = Slot(uri=TEACHINGSCHEMA.pain, name="pain", curie=TEACHINGSCHEMA.curie('pain'),
                   model_uri=TEACHINGSCHEMA.pain, domain=None, range=Optional[float])

slots.item = Slot(uri=TEACHINGSCHEMA.item, name="item", curie=TEACHINGSCHEMA.curie('item'),
                   model_uri=TEACHINGSCHEMA.item, domain=None, range=Optional[str])

slots.fix = Slot(uri=TEACHINGSCHEMA.fix, name="fix", curie=TEACHINGSCHEMA.curie('fix'),
                   model_uri=TEACHINGSCHEMA.fix, domain=None, range=Optional[str])

slots.expectation = Slot(uri=TEACHINGSCHEMA.expectation, name="expectation", curie=TEACHINGSCHEMA.curie('expectation'),
                   model_uri=TEACHINGSCHEMA.expectation, domain=None, range=Optional[str])

slots.examination = Slot(uri=TEACHINGSCHEMA.examination, name="examination", curie=TEACHINGSCHEMA.curie('examination'),
                   model_uri=TEACHINGSCHEMA.examination, domain=None, range=Optional[str])

slots.description = Slot(uri=TEACHINGSCHEMA.description, name="description", curie=TEACHINGSCHEMA.curie('description'),
                   model_uri=TEACHINGSCHEMA.description, domain=None, range=Optional[str])

slots.doi = Slot(uri=TEACHINGSCHEMA.doi, name="doi", curie=TEACHINGSCHEMA.curie('doi'),
                   model_uri=TEACHINGSCHEMA.doi, domain=None, range=Optional[str])

slots.isbn = Slot(uri=TEACHINGSCHEMA.isbn, name="isbn", curie=TEACHINGSCHEMA.curie('isbn'),
                   model_uri=TEACHINGSCHEMA.isbn, domain=None, range=Optional[str])

slots.authors = Slot(uri=TEACHINGSCHEMA.authors, name="authors", curie=TEACHINGSCHEMA.curie('authors'),
                   model_uri=TEACHINGSCHEMA.authors, domain=None, range=Optional[str])

slots.publisher = Slot(uri=TEACHINGSCHEMA.publisher, name="publisher", curie=TEACHINGSCHEMA.curie('publisher'),
                   model_uri=TEACHINGSCHEMA.publisher, domain=None, range=Optional[str])

slots.year = Slot(uri=TEACHINGSCHEMA.year, name="year", curie=TEACHINGSCHEMA.curie('year'),
                   model_uri=TEACHINGSCHEMA.year, domain=None, range=Optional[float])

slots.pdfUrl = Slot(uri=TEACHINGSCHEMA.pdfUrl, name="pdfUrl", curie=TEACHINGSCHEMA.curie('pdfUrl'),
                   model_uri=TEACHINGSCHEMA.pdfUrl, domain=None, range=Optional[Union[str, URI]])

slots.gid = Slot(uri=TEACHINGSCHEMA.gid, name="gid", curie=TEACHINGSCHEMA.curie('gid'),
                   model_uri=TEACHINGSCHEMA.gid, domain=None, range=Optional[str])

slots.url = Slot(uri=TEACHINGSCHEMA.url, name="url", curie=TEACHINGSCHEMA.curie('url'),
                   model_uri=TEACHINGSCHEMA.url, domain=None, range=Optional[str])

slots.chapter__key = Slot(uri=TEACHINGSCHEMA.key, name="chapter__key", curie=TEACHINGSCHEMA.curie('key'),
                   model_uri=TEACHINGSCHEMA.chapter__key, domain=None, range=Optional[str])

slots.chapter__no = Slot(uri=TEACHINGSCHEMA.no, name="chapter__no", curie=TEACHINGSCHEMA.curie('no'),
                   model_uri=TEACHINGSCHEMA.chapter__no, domain=None, range=Optional[str])

slots.chapter__pres_id = Slot(uri=TEACHINGSCHEMA.pres_id, name="chapter__pres_id", curie=TEACHINGSCHEMA.curie('pres_id'),
                   model_uri=TEACHINGSCHEMA.chapter__pres_id, domain=None, range=Optional[str])

slots.chapter__week = Slot(uri=TEACHINGSCHEMA.week, name="chapter__week", curie=TEACHINGSCHEMA.curie('week'),
                   model_uri=TEACHINGSCHEMA.chapter__week, domain=None, range=Optional[str])

slots.chapter__de = Slot(uri=TEACHINGSCHEMA.de, name="chapter__de", curie=TEACHINGSCHEMA.curie('de'),
                   model_uri=TEACHINGSCHEMA.chapter__de, domain=None, range=Optional[str])

slots.chapter__en = Slot(uri=TEACHINGSCHEMA.en, name="chapter__en", curie=TEACHINGSCHEMA.curie('en'),
                   model_uri=TEACHINGSCHEMA.chapter__en, domain=None, range=Optional[str])

slots.presentation__key = Slot(uri=TEACHINGSCHEMA.key, name="presentation__key", curie=TEACHINGSCHEMA.curie('key'),
                   model_uri=TEACHINGSCHEMA.presentation__key, domain=None, range=Optional[str])

slots.presentation__num = Slot(uri=TEACHINGSCHEMA.num, name="presentation__num", curie=TEACHINGSCHEMA.curie('num'),
                   model_uri=TEACHINGSCHEMA.presentation__num, domain=None, range=Optional[str])

slots.presentation__name = Slot(uri=TEACHINGSCHEMA.name, name="presentation__name", curie=TEACHINGSCHEMA.curie('name'),
                   model_uri=TEACHINGSCHEMA.presentation__name, domain=None, range=Optional[str])

slots.presentation__ppt_file = Slot(uri=TEACHINGSCHEMA.ppt_file, name="presentation__ppt_file", curie=TEACHINGSCHEMA.curie('ppt_file'),
                   model_uri=TEACHINGSCHEMA.presentation__ppt_file, domain=None, range=Optional[str])

slots.presentation__sciebo = Slot(uri=TEACHINGSCHEMA.sciebo, name="presentation__sciebo", curie=TEACHINGSCHEMA.curie('sciebo'),
                   model_uri=TEACHINGSCHEMA.presentation__sciebo, domain=None, range=Optional[str])

slots.presentation__gitlab = Slot(uri=TEACHINGSCHEMA.gitlab, name="presentation__gitlab", curie=TEACHINGSCHEMA.curie('gitlab'),
                   model_uri=TEACHINGSCHEMA.presentation__gitlab, domain=None, range=Optional[str])

slots.presentation__moodle = Slot(uri=TEACHINGSCHEMA.moodle, name="presentation__moodle", curie=TEACHINGSCHEMA.curie('moodle'),
                   model_uri=TEACHINGSCHEMA.presentation__moodle, domain=None, range=Optional[str])

slots.learningGoal__key = Slot(uri=TEACHINGSCHEMA.key, name="learningGoal__key", curie=TEACHINGSCHEMA.curie('key'),
                   model_uri=TEACHINGSCHEMA.learningGoal__key, domain=None, range=Optional[str])

slots.learningGoal__qKey = Slot(uri=TEACHINGSCHEMA.qKey, name="learningGoal__qKey", curie=TEACHINGSCHEMA.curie('qKey'),
                   model_uri=TEACHINGSCHEMA.learningGoal__qKey, domain=None, range=Optional[str])

slots.learningGoal__id = Slot(uri=TEACHINGSCHEMA.id, name="learningGoal__id", curie=TEACHINGSCHEMA.curie('id'),
                   model_uri=TEACHINGSCHEMA.learningGoal__id, domain=None, range=Optional[str])

slots.learningGoal__chapter = Slot(uri=TEACHINGSCHEMA.chapter, name="learningGoal__chapter", curie=TEACHINGSCHEMA.curie('chapter'),
                   model_uri=TEACHINGSCHEMA.learningGoal__chapter, domain=None, range=Optional[str])

slots.learningGoal__chapterKey = Slot(uri=TEACHINGSCHEMA.chapterKey, name="learningGoal__chapterKey", curie=TEACHINGSCHEMA.curie('chapterKey'),
                   model_uri=TEACHINGSCHEMA.learningGoal__chapterKey, domain=None, range=Optional[str])

slots.learningGoal__subChapter = Slot(uri=TEACHINGSCHEMA.subChapter, name="learningGoal__subChapter", curie=TEACHINGSCHEMA.curie('subChapter'),
                   model_uri=TEACHINGSCHEMA.learningGoal__subChapter, domain=None, range=Optional[str])

slots.learningGoal__subChapterKey = Slot(uri=TEACHINGSCHEMA.subChapterKey, name="learningGoal__subChapterKey", curie=TEACHINGSCHEMA.curie('subChapterKey'),
                   model_uri=TEACHINGSCHEMA.learningGoal__subChapterKey, domain=None, range=Optional[str])

slots.learningGoal__relevance = Slot(uri=TEACHINGSCHEMA.relevance, name="learningGoal__relevance", curie=TEACHINGSCHEMA.curie('relevance'),
                   model_uri=TEACHINGSCHEMA.learningGoal__relevance, domain=None, range=Optional[str])

slots.learningGoal__keywords = Slot(uri=TEACHINGSCHEMA.keywords, name="learningGoal__keywords", curie=TEACHINGSCHEMA.curie('keywords'),
                   model_uri=TEACHINGSCHEMA.learningGoal__keywords, domain=None, range=Optional[str])

slots.learningGoal__since = Slot(uri=TEACHINGSCHEMA.since, name="learningGoal__since", curie=TEACHINGSCHEMA.curie('since'),
                   model_uri=TEACHINGSCHEMA.learningGoal__since, domain=None, range=Optional[Union[str, XSDDate]])

slots.learningGoal__until = Slot(uri=TEACHINGSCHEMA.until, name="learningGoal__until", curie=TEACHINGSCHEMA.curie('until'),
                   model_uri=TEACHINGSCHEMA.learningGoal__until, domain=None, range=Optional[Union[str, XSDDate]])

slots.keyword__key = Slot(uri=TEACHINGSCHEMA.key, name="keyword__key", curie=TEACHINGSCHEMA.curie('key'),
                   model_uri=TEACHINGSCHEMA.keyword__key, domain=None, range=Optional[str])

slots.keyword__aliases = Slot(uri=TEACHINGSCHEMA.aliases, name="keyword__aliases", curie=TEACHINGSCHEMA.curie('aliases'),
                   model_uri=TEACHINGSCHEMA.keyword__aliases, domain=None, range=Optional[str])

slots.keyword__de = Slot(uri=TEACHINGSCHEMA.de, name="keyword__de", curie=TEACHINGSCHEMA.curie('de'),
                   model_uri=TEACHINGSCHEMA.keyword__de, domain=None, range=Optional[str])

slots.keyword__en = Slot(uri=TEACHINGSCHEMA.en, name="keyword__en", curie=TEACHINGSCHEMA.curie('en'),
                   model_uri=TEACHINGSCHEMA.keyword__en, domain=None, range=Optional[str])

slots.keyword__wikidataid = Slot(uri=TEACHINGSCHEMA.wikidataid, name="keyword__wikidataid", curie=TEACHINGSCHEMA.curie('wikidataid'),
                   model_uri=TEACHINGSCHEMA.keyword__wikidataid, domain=None, range=Optional[str])

slots.keyword__links = Slot(uri=TEACHINGSCHEMA.links, name="keyword__links", curie=TEACHINGSCHEMA.curie('links'),
                   model_uri=TEACHINGSCHEMA.keyword__links, domain=None, range=Optional[str])

slots.keyword__literature = Slot(uri=TEACHINGSCHEMA.literature, name="keyword__literature", curie=TEACHINGSCHEMA.curie('literature'),
                   model_uri=TEACHINGSCHEMA.keyword__literature, domain=None, range=Optional[str])

slots.keyword__urls = Slot(uri=TEACHINGSCHEMA.urls, name="keyword__urls", curie=TEACHINGSCHEMA.curie('urls'),
                   model_uri=TEACHINGSCHEMA.keyword__urls, domain=None, range=Optional[str])

slots.keyword__stackoverflow_tag = Slot(uri=TEACHINGSCHEMA.stackoverflow_tag, name="keyword__stackoverflow_tag", curie=TEACHINGSCHEMA.curie('stackoverflow_tag'),
                   model_uri=TEACHINGSCHEMA.keyword__stackoverflow_tag, domain=None, range=Optional[str])

slots.slide__qkey = Slot(uri=TEACHINGSCHEMA.qkey, name="slide__qkey", curie=TEACHINGSCHEMA.curie('qkey'),
                   model_uri=TEACHINGSCHEMA.slide__qkey, domain=None, range=Optional[str])

slots.slide__pkey = Slot(uri=TEACHINGSCHEMA.pkey, name="slide__pkey", curie=TEACHINGSCHEMA.curie('pkey'),
                   model_uri=TEACHINGSCHEMA.slide__pkey, domain=None, range=Optional[str])

slots.slide__basename = Slot(uri=TEACHINGSCHEMA.basename, name="slide__basename", curie=TEACHINGSCHEMA.curie('basename'),
                   model_uri=TEACHINGSCHEMA.slide__basename, domain=None, range=Optional[str])

slots.slide__page = Slot(uri=TEACHINGSCHEMA.page, name="slide__page", curie=TEACHINGSCHEMA.curie('page'),
                   model_uri=TEACHINGSCHEMA.slide__page, domain=None, range=Optional[str])

slots.slide__name = Slot(uri=TEACHINGSCHEMA.name, name="slide__name", curie=TEACHINGSCHEMA.curie('name'),
                   model_uri=TEACHINGSCHEMA.slide__name, domain=None, range=Optional[str])

slots.slide__title = Slot(uri=TEACHINGSCHEMA.title, name="slide__title", curie=TEACHINGSCHEMA.curie('title'),
                   model_uri=TEACHINGSCHEMA.slide__title, domain=None, range=Optional[str])

slots.slide__keywords = Slot(uri=TEACHINGSCHEMA.keywords, name="slide__keywords", curie=TEACHINGSCHEMA.curie('keywords'),
                   model_uri=TEACHINGSCHEMA.slide__keywords, domain=None, range=Optional[str])

slots.slide__links = Slot(uri=TEACHINGSCHEMA.links, name="slide__links", curie=TEACHINGSCHEMA.curie('links'),
                   model_uri=TEACHINGSCHEMA.slide__links, domain=None, range=Optional[str])

slots.slide__literature = Slot(uri=TEACHINGSCHEMA.literature, name="slide__literature", curie=TEACHINGSCHEMA.curie('literature'),
                   model_uri=TEACHINGSCHEMA.slide__literature, domain=None, range=Optional[str])

slots.slide__learningGoal = Slot(uri=TEACHINGSCHEMA.learningGoal, name="slide__learningGoal", curie=TEACHINGSCHEMA.curie('learningGoal'),
                   model_uri=TEACHINGSCHEMA.slide__learningGoal, domain=None, range=Optional[str])

slots.check__qKey = Slot(uri=TEACHINGSCHEMA.qKey, name="check__qKey", curie=TEACHINGSCHEMA.curie('qKey'),
                   model_uri=TEACHINGSCHEMA.check__qKey, domain=None, range=Optional[str])

slots.check__key = Slot(uri=TEACHINGSCHEMA.key, name="check__key", curie=TEACHINGSCHEMA.curie('key'),
                   model_uri=TEACHINGSCHEMA.check__key, domain=None, range=Optional[str])

slots.check__concept = Slot(uri=TEACHINGSCHEMA.concept, name="check__concept", curie=TEACHINGSCHEMA.curie('concept'),
                   model_uri=TEACHINGSCHEMA.check__concept, domain=None, range=Optional[str])

slots.check__since = Slot(uri=TEACHINGSCHEMA.since, name="check__since", curie=TEACHINGSCHEMA.curie('since'),
                   model_uri=TEACHINGSCHEMA.check__since, domain=None, range=Optional[Union[str, XSDDate]])

slots.check__check = Slot(uri=TEACHINGSCHEMA.check, name="check__check", curie=TEACHINGSCHEMA.curie('check'),
                   model_uri=TEACHINGSCHEMA.check__check, domain=None, range=Optional[str])

slots.check__problem = Slot(uri=TEACHINGSCHEMA.problem, name="check__problem", curie=TEACHINGSCHEMA.curie('problem'),
                   model_uri=TEACHINGSCHEMA.check__problem, domain=None, range=Optional[str])

slots.check__ok = Slot(uri=TEACHINGSCHEMA.ok, name="check__ok", curie=TEACHINGSCHEMA.curie('ok'),
                   model_uri=TEACHINGSCHEMA.check__ok, domain=None, range=Optional[float])

slots.check__total = Slot(uri=TEACHINGSCHEMA.total, name="check__total", curie=TEACHINGSCHEMA.curie('total'),
                   model_uri=TEACHINGSCHEMA.check__total, domain=None, range=Optional[float])

slots.check__pain = Slot(uri=TEACHINGSCHEMA.pain, name="check__pain", curie=TEACHINGSCHEMA.curie('pain'),
                   model_uri=TEACHINGSCHEMA.check__pain, domain=None, range=Optional[float])

slots.issue__item = Slot(uri=TEACHINGSCHEMA.item, name="issue__item", curie=TEACHINGSCHEMA.curie('item'),
                   model_uri=TEACHINGSCHEMA.issue__item, domain=None, range=Optional[str])

slots.issue__check = Slot(uri=TEACHINGSCHEMA.check, name="issue__check", curie=TEACHINGSCHEMA.curie('check'),
                   model_uri=TEACHINGSCHEMA.issue__check, domain=None, range=Optional[str])

slots.issue__fix = Slot(uri=TEACHINGSCHEMA.fix, name="issue__fix", curie=TEACHINGSCHEMA.curie('fix'),
                   model_uri=TEACHINGSCHEMA.issue__fix, domain=None, range=Optional[str])

slots.relevanceLevel__id = Slot(uri=TEACHINGSCHEMA.id, name="relevanceLevel__id", curie=TEACHINGSCHEMA.curie('id'),
                   model_uri=TEACHINGSCHEMA.relevanceLevel__id, domain=None, range=Optional[str])

slots.relevanceLevel__expectation = Slot(uri=TEACHINGSCHEMA.expectation, name="relevanceLevel__expectation", curie=TEACHINGSCHEMA.curie('expectation'),
                   model_uri=TEACHINGSCHEMA.relevanceLevel__expectation, domain=None, range=Optional[str])

slots.relevanceLevel__examination = Slot(uri=TEACHINGSCHEMA.examination, name="relevanceLevel__examination", curie=TEACHINGSCHEMA.curie('examination'),
                   model_uri=TEACHINGSCHEMA.relevanceLevel__examination, domain=None, range=Optional[str])

slots.relevanceLevel__description = Slot(uri=TEACHINGSCHEMA.description, name="relevanceLevel__description", curie=TEACHINGSCHEMA.curie('description'),
                   model_uri=TEACHINGSCHEMA.relevanceLevel__description, domain=None, range=Optional[str])

slots.publication__id = Slot(uri=TEACHINGSCHEMA.id, name="publication__id", curie=TEACHINGSCHEMA.curie('id'),
                   model_uri=TEACHINGSCHEMA.publication__id, domain=None, range=Optional[str])

slots.publication__doi = Slot(uri=TEACHINGSCHEMA.doi, name="publication__doi", curie=TEACHINGSCHEMA.curie('doi'),
                   model_uri=TEACHINGSCHEMA.publication__doi, domain=None, range=Optional[str])

slots.publication__wikidataid = Slot(uri=TEACHINGSCHEMA.wikidataid, name="publication__wikidataid", curie=TEACHINGSCHEMA.curie('wikidataid'),
                   model_uri=TEACHINGSCHEMA.publication__wikidataid, domain=None, range=Optional[str])

slots.publication__isbn = Slot(uri=TEACHINGSCHEMA.isbn, name="publication__isbn", curie=TEACHINGSCHEMA.curie('isbn'),
                   model_uri=TEACHINGSCHEMA.publication__isbn, domain=None, range=Optional[str])

slots.publication__title = Slot(uri=TEACHINGSCHEMA.title, name="publication__title", curie=TEACHINGSCHEMA.curie('title'),
                   model_uri=TEACHINGSCHEMA.publication__title, domain=None, range=Optional[str])

slots.publication__authors = Slot(uri=TEACHINGSCHEMA.authors, name="publication__authors", curie=TEACHINGSCHEMA.curie('authors'),
                   model_uri=TEACHINGSCHEMA.publication__authors, domain=None, range=Optional[str])

slots.publication__publisher = Slot(uri=TEACHINGSCHEMA.publisher, name="publication__publisher", curie=TEACHINGSCHEMA.curie('publisher'),
                   model_uri=TEACHINGSCHEMA.publication__publisher, domain=None, range=Optional[str])

slots.publication__year = Slot(uri=TEACHINGSCHEMA.year, name="publication__year", curie=TEACHINGSCHEMA.curie('year'),
                   model_uri=TEACHINGSCHEMA.publication__year, domain=None, range=Optional[float])

slots.publication__pdfUrl = Slot(uri=TEACHINGSCHEMA.pdfUrl, name="publication__pdfUrl", curie=TEACHINGSCHEMA.curie('pdfUrl'),
                   model_uri=TEACHINGSCHEMA.publication__pdfUrl, domain=None, range=Optional[Union[str, URI]])

slots.sheet__gid = Slot(uri=TEACHINGSCHEMA.gid, name="sheet__gid", curie=TEACHINGSCHEMA.curie('gid'),
                   model_uri=TEACHINGSCHEMA.sheet__gid, domain=None, range=Optional[str])

slots.sheet__concept = Slot(uri=TEACHINGSCHEMA.concept, name="sheet__concept", curie=TEACHINGSCHEMA.curie('concept'),
                   model_uri=TEACHINGSCHEMA.sheet__concept, domain=None, range=Optional[str])

slots.sheet__url = Slot(uri=TEACHINGSCHEMA.url, name="sheet__url", curie=TEACHINGSCHEMA.curie('url'),
                   model_uri=TEACHINGSCHEMA.sheet__url, domain=None, range=Optional[str])



# Auto generated from city.yaml by pythongen.py version: 0.9.0
# Generation date: 2023-03-08T09:54:31
# Schema: GeneralContext
#
# id: GeneralContext
# description:
# license:

import dataclasses
import sys
import re
from jsonasobj2 import JsonObj, as_dict
from typing import Optional, List, Union, Dict, ClassVar, Any
from dataclasses import dataclass
from linkml_runtime.linkml_model.meta import EnumDefinition, PermissibleValue, PvFormulaOptions

from linkml_runtime.utils.slot import Slot
from linkml_runtime.utils.metamodelcore import empty_list, empty_dict, bnode
from linkml_runtime.utils.yamlutils import YAMLRoot, extended_str, extended_float, extended_int
from linkml_runtime.utils.dataclass_extensions_376 import dataclasses_init_fn_with_kwargs
from linkml_runtime.utils.formatutils import camelcase, underscore, sfx
from linkml_runtime.utils.enumerations import EnumDefinitionImpl
from rdflib import Namespace, URIRef
from linkml_runtime.utils.curienamespace import CurieNamespace
from linkml_runtime.utils.metamodelcore import Bool, Curie, Decimal, ElementIdentifier, NCName, NodeIdentifier, URI, URIorCURIE, XSDDate, XSDDateTime, XSDTime

metamodel_version = "1.7.0"
version = None

# Overwrite dataclasses _init_fn to add **kwargs in __init__
dataclasses._init_fn = dataclasses_init_fn_with_kwargs

# Namespaces
GENERALCONTEXT = CurieNamespace('GeneralContext', 'GeneralContext/')
LINKML = CurieNamespace('linkml', 'https://w3id.org/linkml/')
SCHEMA = CurieNamespace('schema', 'http://schema.org/')
SHEX = CurieNamespace('shex', 'http://www.w3.org/ns/shex#')
XSD = CurieNamespace('xsd', 'http://www.w3.org/2001/XMLSchema#')
DEFAULT_ = GENERALCONTEXT


# Types
class String(str):
    """ A character string """
    type_class_uri = XSD.string
    type_class_curie = "xsd:string"
    type_name = "string"
    type_model_uri = GENERALCONTEXT.String


class Integer(int):
    """ An integer """
    type_class_uri = XSD.integer
    type_class_curie = "xsd:integer"
    type_name = "integer"
    type_model_uri = GENERALCONTEXT.Integer


class Boolean(Bool):
    """ A binary (true or false) value """
    type_class_uri = XSD.boolean
    type_class_curie = "xsd:boolean"
    type_name = "boolean"
    type_model_uri = GENERALCONTEXT.Boolean


class Float(float):
    """ A real number that conforms to the xsd:float specification """
    type_class_uri = XSD.float
    type_class_curie = "xsd:float"
    type_name = "float"
    type_model_uri = GENERALCONTEXT.Float


class Double(float):
    """ A real number that conforms to the xsd:double specification """
    type_class_uri = XSD.double
    type_class_curie = "xsd:double"
    type_name = "double"
    type_model_uri = GENERALCONTEXT.Double


class Decimal(Decimal):
    """ A real number with arbitrary precision that conforms to the xsd:decimal specification """
    type_class_uri = XSD.decimal
    type_class_curie = "xsd:decimal"
    type_name = "decimal"
    type_model_uri = GENERALCONTEXT.Decimal


class Time(XSDTime):
    """ A time object represents a (local) time of day, independent of any particular day """
    type_class_uri = XSD.dateTime
    type_class_curie = "xsd:dateTime"
    type_name = "time"
    type_model_uri = GENERALCONTEXT.Time


class Date(XSDDate):
    """ a date (year, month and day) in an idealized calendar """
    type_class_uri = XSD.date
    type_class_curie = "xsd:date"
    type_name = "date"
    type_model_uri = GENERALCONTEXT.Date


class Datetime(XSDDateTime):
    """ The combination of a date and time """
    type_class_uri = XSD.dateTime
    type_class_curie = "xsd:dateTime"
    type_name = "datetime"
    type_model_uri = GENERALCONTEXT.Datetime


class DateOrDatetime(str):
    """ Either a date or a datetime """
    type_class_uri = LINKML.DateOrDatetime
    type_class_curie = "linkml:DateOrDatetime"
    type_name = "date_or_datetime"
    type_model_uri = GENERALCONTEXT.DateOrDatetime


class Uriorcurie(URIorCURIE):
    """ a URI or a CURIE """
    type_class_uri = XSD.anyURI
    type_class_curie = "xsd:anyURI"
    type_name = "uriorcurie"
    type_model_uri = GENERALCONTEXT.Uriorcurie


class Curie(Curie):
    """ a compact URI """
    type_class_uri = XSD.string
    type_class_curie = "xsd:string"
    type_name = "curie"
    type_model_uri = GENERALCONTEXT.Curie


class Uri(URI):
    """ a complete URI """
    type_class_uri = XSD.anyURI
    type_class_curie = "xsd:anyURI"
    type_name = "uri"
    type_model_uri = GENERALCONTEXT.Uri


class Ncname(NCName):
    """ Prefix part of CURIE """
    type_class_uri = XSD.string
    type_class_curie = "xsd:string"
    type_name = "ncname"
    type_model_uri = GENERALCONTEXT.Ncname


class Objectidentifier(ElementIdentifier):
    """ A URI or CURIE that represents an object in the model. """
    type_class_uri = SHEX.iri
    type_class_curie = "shex:iri"
    type_name = "objectidentifier"
    type_model_uri = GENERALCONTEXT.Objectidentifier


class Nodeidentifier(NodeIdentifier):
    """ A URI, CURIE or BNODE that represents a node in a model. """
    type_class_uri = SHEX.nonLiteral
    type_class_curie = "shex:nonLiteral"
    type_name = "nodeidentifier"
    type_model_uri = GENERALCONTEXT.Nodeidentifier


# Class references



@dataclass
class City(YAMLRoot):
    """
    I represent a City like Berlin, New York or Tokyo
    """
    _inherited_slots: ClassVar[List[str]] = []

    class_class_uri: ClassVar[URIRef] = GENERALCONTEXT.City
    class_class_curie: ClassVar[str] = "GeneralContext:City"
    class_name: ClassVar[str] = "City"
    class_model_uri: ClassVar[URIRef] = GENERALCONTEXT.City

    name: Optional[str] = None
    webpage: Optional[Union[str, URI]] = None
    wikipedia_url: Optional[Union[str, URI]] = None
    Population: Optional[float] = None

    def __post_init__(self, *_: List[str], **kwargs: Dict[str, Any]):
        if self.name is not None and not isinstance(self.name, str):
            self.name = str(self.name)

        if self.webpage is not None and not isinstance(self.webpage, URI):
            self.webpage = URI(self.webpage)

        if self.wikipedia_url is not None and not isinstance(self.wikipedia_url, URI):
            self.wikipedia_url = URI(self.wikipedia_url)

        if self.Population is not None and not isinstance(self.Population, float):
            self.Population = float(self.Population)

        super().__post_init__(**kwargs)


# Enumerations


# Slots
class slots:
    pass

slots.name = Slot(uri=GENERALCONTEXT.name, name="name", curie=GENERALCONTEXT.curie('name'),
                   model_uri=GENERALCONTEXT.name, domain=None, range=Optional[str])

slots.webpage = Slot(uri=GENERALCONTEXT.webpage, name="webpage", curie=GENERALCONTEXT.curie('webpage'),
                   model_uri=GENERALCONTEXT.webpage, domain=None, range=Optional[Union[str, URI]])

slots.wikipedia_url = Slot(uri=GENERALCONTEXT.wikipedia_url, name="wikipedia_url", curie=GENERALCONTEXT.curie('wikipedia_url'),
                   model_uri=GENERALCONTEXT.wikipedia_url, domain=None, range=Optional[Union[str, URI]])

slots.Population = Slot(uri=GENERALCONTEXT.Population, name="Population", curie=GENERALCONTEXT.curie('Population'),
                   model_uri=GENERALCONTEXT.Population, domain=None, range=Optional[float])

slots.city__name = Slot(uri=GENERALCONTEXT.name, name="city__name", curie=GENERALCONTEXT.curie('name'),
                   model_uri=GENERALCONTEXT.city__name, domain=None, range=Optional[str])

slots.city__webpage = Slot(uri=GENERALCONTEXT.webpage, name="city__webpage", curie=GENERALCONTEXT.curie('webpage'),
                   model_uri=GENERALCONTEXT.city__webpage, domain=None, range=Optional[Union[str, URI]])

slots.city__wikipedia_url = Slot(uri=GENERALCONTEXT.wikipedia_url, name="city__wikipedia_url", curie=GENERALCONTEXT.curie('wikipedia_url'),
                   model_uri=GENERALCONTEXT.city__wikipedia_url, domain=None, range=Optional[Union[str, URI]])

slots.city__Population = Slot(uri=GENERALCONTEXT.Population, name="city__Population", curie=GENERALCONTEXT.curie('Population'),
                   model_uri=GENERALCONTEXT.city__Population, domain=None, range=Optional[float])



'''
Created on 2023-02-25

@author: wf
'''
from tests.basetest import Basetest
from meta.metamodel import Context

class TestSiDIF(Basetest):
    """
    test plantuml handling
    """
    def testIssue18_readingFromSiDIF_input(self):
        '''
        https://github.com/WolfgangFahl/py-sidif/issues/4
        add fromSiDIF_input api to allow reading Context from url or file_path via API
        '''
        debug=self.debug
        #debug=True
        for url in ["https://raw.githubusercontent.com/WolfgangFahl/pyMetaModel/main/examples/city/city.sidif"]:
            context,error,errMsg=Context.fromSiDIF_input(url,debug=debug)
            self.assertFalse(error)
            self.assertIsNone(errMsg)
            self.assertIsNotNone(context)
            if debug:
                print(context)





# Auto generated from scientific-events.yaml by pythongen.py version: 0.9.0
# Generation date: 2023-03-08T09:54:23
# Schema: CrSchema
#
# id: CrSchema
# description:
# license:

import dataclasses
import sys
import re
from jsonasobj2 import JsonObj, as_dict
from typing import Optional, List, Union, Dict, ClassVar, Any
from dataclasses import dataclass
from linkml_runtime.linkml_model.meta import EnumDefinition, PermissibleValue, PvFormulaOptions

from linkml_runtime.utils.slot import Slot
from linkml_runtime.utils.metamodelcore import empty_list, empty_dict, bnode
from linkml_runtime.utils.yamlutils import YAMLRoot, extended_str, extended_float, extended_int
from linkml_runtime.utils.dataclass_extensions_376 import dataclasses_init_fn_with_kwargs
from linkml_runtime.utils.formatutils import camelcase, underscore, sfx
from linkml_runtime.utils.enumerations import EnumDefinitionImpl
from rdflib import Namespace, URIRef
from linkml_runtime.utils.curienamespace import CurieNamespace
from linkml_runtime.utils.metamodelcore import Bool, Curie, Decimal, ElementIdentifier, NCName, NodeIdentifier, URI, URIorCURIE, XSDDate, XSDDateTime, XSDTime

metamodel_version = "1.7.0"
version = None

# Overwrite dataclasses _init_fn to add **kwargs in __init__
dataclasses._init_fn = dataclasses_init_fn_with_kwargs

# Namespaces
CRSCHEMA = CurieNamespace('CrSchema', 'CrSchema/')
LINKML = CurieNamespace('linkml', 'https://w3id.org/linkml/')
SCHEMA = CurieNamespace('schema', 'http://schema.org/')
SHEX = CurieNamespace('shex', 'http://www.w3.org/ns/shex#')
XSD = CurieNamespace('xsd', 'http://www.w3.org/2001/XMLSchema#')
DEFAULT_ = CRSCHEMA


# Types
class String(str):
    """ A character string """
    type_class_uri = XSD.string
    type_class_curie = "xsd:string"
    type_name = "string"
    type_model_uri = CRSCHEMA.String


class Integer(int):
    """ An integer """
    type_class_uri = XSD.integer
    type_class_curie = "xsd:integer"
    type_name = "integer"
    type_model_uri = CRSCHEMA.Integer


class Boolean(Bool):
    """ A binary (true or false) value """
    type_class_uri = XSD.boolean
    type_class_curie = "xsd:boolean"
    type_name = "boolean"
    type_model_uri = CRSCHEMA.Boolean


class Float(float):
    """ A real number that conforms to the xsd:float specification """
    type_class_uri = XSD.float
    type_class_curie = "xsd:float"
    type_name = "float"
    type_model_uri = CRSCHEMA.Float


class Double(float):
    """ A real number that conforms to the xsd:double specification """
    type_class_uri = XSD.double
    type_class_curie = "xsd:double"
    type_name = "double"
    type_model_uri = CRSCHEMA.Double


class Decimal(Decimal):
    """ A real number with arbitrary precision that conforms to the xsd:decimal specification """
    type_class_uri = XSD.decimal
    type_class_curie = "xsd:decimal"
    type_name = "decimal"
    type_model_uri = CRSCHEMA.Decimal


class Time(XSDTime):
    """ A time object represents a (local) time of day, independent of any particular day """
    type_class_uri = XSD.dateTime
    type_class_curie = "xsd:dateTime"
    type_name = "time"
    type_model_uri = CRSCHEMA.Time


class Date(XSDDate):
    """ a date (year, month and day) in an idealized calendar """
    type_class_uri = XSD.date
    type_class_curie = "xsd:date"
    type_name = "date"
    type_model_uri = CRSCHEMA.Date


class Datetime(XSDDateTime):
    """ The combination of a date and time """
    type_class_uri = XSD.dateTime
    type_class_curie = "xsd:dateTime"
    type_name = "datetime"
    type_model_uri = CRSCHEMA.Datetime


class DateOrDatetime(str):
    """ Either a date or a datetime """
    type_class_uri = LINKML.DateOrDatetime
    type_class_curie = "linkml:DateOrDatetime"
    type_name = "date_or_datetime"
    type_model_uri = CRSCHEMA.DateOrDatetime


class Uriorcurie(URIorCURIE):
    """ a URI or a CURIE """
    type_class_uri = XSD.anyURI
    type_class_curie = "xsd:anyURI"
    type_name = "uriorcurie"
    type_model_uri = CRSCHEMA.Uriorcurie


class Curie(Curie):
    """ a compact URI """
    type_class_uri = XSD.string
    type_class_curie = "xsd:string"
    type_name = "curie"
    type_model_uri = CRSCHEMA.Curie


class Uri(URI):
    """ a complete URI """
    type_class_uri = XSD.anyURI
    type_class_curie = "xsd:anyURI"
    type_name = "uri"
    type_model_uri = CRSCHEMA.Uri


class Ncname(NCName):
    """ Prefix part of CURIE """
    type_class_uri = XSD.string
    type_class_curie = "xsd:string"
    type_name = "ncname"
    type_model_uri = CRSCHEMA.Ncname


class Objectidentifier(ElementIdentifier):
    """ A URI or CURIE that represents an object in the model. """
    type_class_uri = SHEX.iri
    type_class_curie = "shex:iri"
    type_name = "objectidentifier"
    type_model_uri = CRSCHEMA.Objectidentifier


class Nodeidentifier(NodeIdentifier):
    """ A URI, CURIE or BNODE that represents a node in a model. """
    type_class_uri = SHEX.nonLiteral
    type_class_curie = "shex:nonLiteral"
    type_name = "nodeidentifier"
    type_model_uri = CRSCHEMA.Nodeidentifier


# Class references



@dataclass
class EventSeries(YAMLRoot):
    """
    a series of Events
    """
    _inherited_slots: ClassVar[List[str]] = []

    class_class_uri: ClassVar[URIRef] = CRSCHEMA.EventSeries
    class_class_curie: ClassVar[str] = "CrSchema:EventSeries"
    class_name: ClassVar[str] = "EventSeries"
    class_model_uri: ClassVar[URIRef] = CRSCHEMA.EventSeries

    acronym: Optional[str] = None
    description: Optional[str] = None
    title: Optional[str] = None
    wikidataid: Optional[str] = None
    homepage: Optional[str] = None

    def __post_init__(self, *_: List[str], **kwargs: Dict[str, Any]):
        if self.acronym is not None and not isinstance(self.acronym, str):
            self.acronym = str(self.acronym)

        if self.description is not None and not isinstance(self.description, str):
            self.description = str(self.description)

        if self.title is not None and not isinstance(self.title, str):
            self.title = str(self.title)

        if self.wikidataid is not None and not isinstance(self.wikidataid, str):
            self.wikidataid = str(self.wikidataid)

        if self.homepage is not None and not isinstance(self.homepage, str):
            self.homepage = str(self.homepage)

        super().__post_init__(**kwargs)


@dataclass
class Event(YAMLRoot):
    """
    a meeting of researchers at a specific time and place
    """
    _inherited_slots: ClassVar[List[str]] = []

    class_class_uri: ClassVar[URIRef] = CRSCHEMA.Event
    class_class_curie: ClassVar[str] = "CrSchema:Event"
    class_name: ClassVar[str] = "Event"
    class_model_uri: ClassVar[URIRef] = CRSCHEMA.Event

    acronym: Optional[str] = None
    description: Optional[str] = None
    title: Optional[str] = None
    homepage: Optional[str] = None
    wikidataid: Optional[str] = None
    series: Optional[Union[dict, EventSeries]] = None
    city: Optional[Union[dict, "City"]] = None
    country: Optional[Union[dict, "Country"]] = None
    region: Optional[Union[dict, "Region"]] = None

    def __post_init__(self, *_: List[str], **kwargs: Dict[str, Any]):
        if self.acronym is not None and not isinstance(self.acronym, str):
            self.acronym = str(self.acronym)

        if self.description is not None and not isinstance(self.description, str):
            self.description = str(self.description)

        if self.title is not None and not isinstance(self.title, str):
            self.title = str(self.title)

        if self.homepage is not None and not isinstance(self.homepage, str):
            self.homepage = str(self.homepage)

        if self.wikidataid is not None and not isinstance(self.wikidataid, str):
            self.wikidataid = str(self.wikidataid)

        if self.series is not None and not isinstance(self.series, EventSeries):
            self.series = EventSeries(**as_dict(self.series))

        if self.city is not None and not isinstance(self.city, City):
            self.city = City(**as_dict(self.city))

        if self.country is not None and not isinstance(self.country, Country):
            self.country = Country(**as_dict(self.country))

        if self.region is not None and not isinstance(self.region, Region):
            self.region = Region(**as_dict(self.region))

        super().__post_init__(**kwargs)


@dataclass
class City(YAMLRoot):
    """
    large permanent human settlement
    """
    _inherited_slots: ClassVar[List[str]] = []

    class_class_uri: ClassVar[URIRef] = CRSCHEMA.City
    class_class_curie: ClassVar[str] = "CrSchema:City"
    class_name: ClassVar[str] = "City"
    class_model_uri: ClassVar[URIRef] = CRSCHEMA.City

    name: Optional[str] = None
    wikidataid: Optional[str] = None

    def __post_init__(self, *_: List[str], **kwargs: Dict[str, Any]):
        if self.name is not None and not isinstance(self.name, str):
            self.name = str(self.name)

        if self.wikidataid is not None and not isinstance(self.wikidataid, str):
            self.wikidataid = str(self.wikidataid)

        super().__post_init__(**kwargs)


@dataclass
class Country(YAMLRoot):
    """
    distinct region in geography; a broad term that can include political divisions or regions associated with
    distinct political characteristics
    """
    _inherited_slots: ClassVar[List[str]] = []

    class_class_uri: ClassVar[URIRef] = CRSCHEMA.Country
    class_class_curie: ClassVar[str] = "CrSchema:Country"
    class_name: ClassVar[str] = "Country"
    class_model_uri: ClassVar[URIRef] = CRSCHEMA.Country

    name: Optional[str] = None
    wikidataid: Optional[str] = None

    def __post_init__(self, *_: List[str], **kwargs: Dict[str, Any]):
        if self.name is not None and not isinstance(self.name, str):
            self.name = str(self.name)

        if self.wikidataid is not None and not isinstance(self.wikidataid, str):
            self.wikidataid = str(self.wikidataid)

        super().__post_init__(**kwargs)


@dataclass
class Region(YAMLRoot):
    """
    first-level administrative country subdivision e.g. Region/province
    """
    _inherited_slots: ClassVar[List[str]] = []

    class_class_uri: ClassVar[URIRef] = CRSCHEMA.Region
    class_class_curie: ClassVar[str] = "CrSchema:Region"
    class_name: ClassVar[str] = "Region"
    class_model_uri: ClassVar[URIRef] = CRSCHEMA.Region

    name: Optional[str] = None
    wikidataid: Optional[str] = None

    def __post_init__(self, *_: List[str], **kwargs: Dict[str, Any]):
        if self.name is not None and not isinstance(self.name, str):
            self.name = str(self.name)

        if self.wikidataid is not None and not isinstance(self.wikidataid, str):
            self.wikidataid = str(self.wikidataid)

        super().__post_init__(**kwargs)


@dataclass
class Proceedings(YAMLRoot):
    """
    Proceedings are a collection of papers mostly documenting the results of an academic event
    """
    _inherited_slots: ClassVar[List[str]] = []

    class_class_uri: ClassVar[URIRef] = CRSCHEMA.Proceedings
    class_class_curie: ClassVar[str] = "CrSchema:Proceedings"
    class_name: ClassVar[str] = "Proceedings"
    class_model_uri: ClassVar[URIRef] = CRSCHEMA.Proceedings

    wikidataid: Optional[str] = None
    title: Optional[str] = None

    def __post_init__(self, *_: List[str], **kwargs: Dict[str, Any]):
        if self.wikidataid is not None and not isinstance(self.wikidataid, str):
            self.wikidataid = str(self.wikidataid)

        if self.title is not None and not isinstance(self.title, str):
            self.title = str(self.title)

        super().__post_init__(**kwargs)


@dataclass
class Scholar(YAMLRoot):
    """
    scholar/researcher: person who engages in research, professionally or otherwise
    """
    _inherited_slots: ClassVar[List[str]] = []

    class_class_uri: ClassVar[URIRef] = CRSCHEMA.Scholar
    class_class_curie: ClassVar[str] = "CrSchema:Scholar"
    class_name: ClassVar[str] = "Scholar"
    class_model_uri: ClassVar[URIRef] = CRSCHEMA.Scholar

    wikiDataId: Optional[str] = None
    name: Optional[str] = None
    firstName: Optional[str] = None
    description: Optional[str] = None
    homepage: Optional[str] = None
    orcid: Optional[str] = None
    dblpId: Optional[str] = None
    googleScholarUser: Optional[str] = None
    linkedInId: Optional[str] = None
    researchGate: Optional[str] = None
    gndId: Optional[str] = None
    smartCRMId: Optional[str] = None

    def __post_init__(self, *_: List[str], **kwargs: Dict[str, Any]):
        if self.wikiDataId is not None and not isinstance(self.wikiDataId, str):
            self.wikiDataId = str(self.wikiDataId)

        if self.name is not None and not isinstance(self.name, str):
            self.name = str(self.name)

        if self.firstName is not None and not isinstance(self.firstName, str):
            self.firstName = str(self.firstName)

        if self.description is not None and not isinstance(self.description, str):
            self.description = str(self.description)

        if self.homepage is not None and not isinstance(self.homepage, str):
            self.homepage = str(self.homepage)

        if self.orcid is not None and not isinstance(self.orcid, str):
            self.orcid = str(self.orcid)

        if self.dblpId is not None and not isinstance(self.dblpId, str):
            self.dblpId = str(self.dblpId)

        if self.googleScholarUser is not None and not isinstance(self.googleScholarUser, str):
            self.googleScholarUser = str(self.googleScholarUser)

        if self.linkedInId is not None and not isinstance(self.linkedInId, str):
            self.linkedInId = str(self.linkedInId)

        if self.researchGate is not None and not isinstance(self.researchGate, str):
            self.researchGate = str(self.researchGate)

        if self.gndId is not None and not isinstance(self.gndId, str):
            self.gndId = str(self.gndId)

        if self.smartCRMId is not None and not isinstance(self.smartCRMId, str):
            self.smartCRMId = str(self.smartCRMId)

        super().__post_init__(**kwargs)


@dataclass
class Institution(YAMLRoot):
    """
    structure or mechanism of social order and cooperation governing the behaviour of a set of individuals within a
    given community
    """
    _inherited_slots: ClassVar[List[str]] = []

    class_class_uri: ClassVar[URIRef] = CRSCHEMA.Institution
    class_class_curie: ClassVar[str] = "CrSchema:Institution"
    class_name: ClassVar[str] = "Institution"
    class_model_uri: ClassVar[URIRef] = CRSCHEMA.Institution

    title: Optional[str] = None
    wikidataid: Optional[str] = None
    homepage: Optional[str] = None
    smartCRMId: Optional[str] = None
    scholar: Optional[Union[dict, Scholar]] = None

    def __post_init__(self, *_: List[str], **kwargs: Dict[str, Any]):
        if self.title is not None and not isinstance(self.title, str):
            self.title = str(self.title)

        if self.wikidataid is not None and not isinstance(self.wikidataid, str):
            self.wikidataid = str(self.wikidataid)

        if self.homepage is not None and not isinstance(self.homepage, str):
            self.homepage = str(self.homepage)

        if self.smartCRMId is not None and not isinstance(self.smartCRMId, str):
            self.smartCRMId = str(self.smartCRMId)

        if self.scholar is not None and not isinstance(self.scholar, Scholar):
            self.scholar = Scholar(**as_dict(self.scholar))

        super().__post_init__(**kwargs)


@dataclass
class Paper(YAMLRoot):
    """
    A paper is e.g. a scholarly article
    """
    _inherited_slots: ClassVar[List[str]] = []

    class_class_uri: ClassVar[URIRef] = CRSCHEMA.Paper
    class_class_curie: ClassVar[str] = "CrSchema:Paper"
    class_name: ClassVar[str] = "Paper"
    class_model_uri: ClassVar[URIRef] = CRSCHEMA.Paper

    description: Optional[str] = None
    id: Optional[str] = None
    wikidataid: Optional[str] = None
    title: Optional[str] = None
    authors: Optional[str] = None
    pdfUrl: Optional[Union[str, URI]] = None

    def __post_init__(self, *_: List[str], **kwargs: Dict[str, Any]):
        if self.description is not None and not isinstance(self.description, str):
            self.description = str(self.description)

        if self.id is not None and not isinstance(self.id, str):
            self.id = str(self.id)

        if self.wikidataid is not None and not isinstance(self.wikidataid, str):
            self.wikidataid = str(self.wikidataid)

        if self.title is not None and not isinstance(self.title, str):
            self.title = str(self.title)

        if self.authors is not None and not isinstance(self.authors, str):
            self.authors = str(self.authors)

        if self.pdfUrl is not None and not isinstance(self.pdfUrl, URI):
            self.pdfUrl = URI(self.pdfUrl)

        super().__post_init__(**kwargs)


# Enumerations


# Slots
class slots:
    pass

slots.acronym = Slot(uri=CRSCHEMA.acronym, name="acronym", curie=CRSCHEMA.curie('acronym'),
                   model_uri=CRSCHEMA.acronym, domain=None, range=Optional[str])

slots.description = Slot(uri=CRSCHEMA.description, name="description", curie=CRSCHEMA.curie('description'),
                   model_uri=CRSCHEMA.description, domain=None, range=Optional[str])

slots.title = Slot(uri=CRSCHEMA.title, name="title", curie=CRSCHEMA.curie('title'),
                   model_uri=CRSCHEMA.title, domain=None, range=Optional[str])

slots.wikidataid = Slot(uri=CRSCHEMA.wikidataid, name="wikidataid", curie=CRSCHEMA.curie('wikidataid'),
                   model_uri=CRSCHEMA.wikidataid, domain=None, range=Optional[str])

slots.homepage = Slot(uri=CRSCHEMA.homepage, name="homepage", curie=CRSCHEMA.curie('homepage'),
                   model_uri=CRSCHEMA.homepage, domain=None, range=Optional[str])

slots.series = Slot(uri=CRSCHEMA.series, name="series", curie=CRSCHEMA.curie('series'),
                   model_uri=CRSCHEMA.series, domain=None, range=Optional[Union[dict, EventSeries]])

slots.city = Slot(uri=CRSCHEMA.city, name="city", curie=CRSCHEMA.curie('city'),
                   model_uri=CRSCHEMA.city, domain=None, range=Optional[Union[dict, City]])

slots.country = Slot(uri=CRSCHEMA.country, name="country", curie=CRSCHEMA.curie('country'),
                   model_uri=CRSCHEMA.country, domain=None, range=Optional[Union[dict, Country]])

slots.region = Slot(uri=CRSCHEMA.region, name="region", curie=CRSCHEMA.curie('region'),
                   model_uri=CRSCHEMA.region, domain=None, range=Optional[Union[dict, Region]])

slots.name = Slot(uri=CRSCHEMA.name, name="name", curie=CRSCHEMA.curie('name'),
                   model_uri=CRSCHEMA.name, domain=None, range=Optional[str])

slots.wikiDataId = Slot(uri=CRSCHEMA.wikiDataId, name="wikiDataId", curie=CRSCHEMA.curie('wikiDataId'),
                   model_uri=CRSCHEMA.wikiDataId, domain=None, range=Optional[str])

slots.firstName = Slot(uri=CRSCHEMA.firstName, name="firstName", curie=CRSCHEMA.curie('firstName'),
                   model_uri=CRSCHEMA.firstName, domain=None, range=Optional[str])

slots.orcid = Slot(uri=CRSCHEMA.orcid, name="orcid", curie=CRSCHEMA.curie('orcid'),
                   model_uri=CRSCHEMA.orcid, domain=None, range=Optional[str])

slots.dblpId = Slot(uri=CRSCHEMA.dblpId, name="dblpId", curie=CRSCHEMA.curie('dblpId'),
                   model_uri=CRSCHEMA.dblpId, domain=None, range=Optional[str])

slots.googleScholarUser = Slot(uri=CRSCHEMA.googleScholarUser, name="googleScholarUser", curie=CRSCHEMA.curie('googleScholarUser'),
                   model_uri=CRSCHEMA.googleScholarUser, domain=None, range=Optional[str])

slots.linkedInId = Slot(uri=CRSCHEMA.linkedInId, name="linkedInId", curie=CRSCHEMA.curie('linkedInId'),
                   model_uri=CRSCHEMA.linkedInId, domain=None, range=Optional[str])

slots.researchGate = Slot(uri=CRSCHEMA.researchGate, name="researchGate", curie=CRSCHEMA.curie('researchGate'),
                   model_uri=CRSCHEMA.researchGate, domain=None, range=Optional[str])

slots.gndId = Slot(uri=CRSCHEMA.gndId, name="gndId", curie=CRSCHEMA.curie('gndId'),
                   model_uri=CRSCHEMA.gndId, domain=None, range=Optional[str])

slots.smartCRMId = Slot(uri=CRSCHEMA.smartCRMId, name="smartCRMId", curie=CRSCHEMA.curie('smartCRMId'),
                   model_uri=CRSCHEMA.smartCRMId, domain=None, range=Optional[str])

slots.scholar = Slot(uri=CRSCHEMA.scholar, name="scholar", curie=CRSCHEMA.curie('scholar'),
                   model_uri=CRSCHEMA.scholar, domain=None, range=Optional[Union[dict, Scholar]])

slots.id = Slot(uri=CRSCHEMA.id, name="id", curie=CRSCHEMA.curie('id'),
                   model_uri=CRSCHEMA.id, domain=None, range=Optional[str])

slots.authors = Slot(uri=CRSCHEMA.authors, name="authors", curie=CRSCHEMA.curie('authors'),
                   model_uri=CRSCHEMA.authors, domain=None, range=Optional[str])

slots.pdfUrl = Slot(uri=CRSCHEMA.pdfUrl, name="pdfUrl", curie=CRSCHEMA.curie('pdfUrl'),
                   model_uri=CRSCHEMA.pdfUrl, domain=None, range=Optional[Union[str, URI]])

slots.eventSeries__acronym = Slot(uri=CRSCHEMA.acronym, name="eventSeries__acronym", curie=CRSCHEMA.curie('acronym'),
                   model_uri=CRSCHEMA.eventSeries__acronym, domain=None, range=Optional[str])

slots.eventSeries__description = Slot(uri=CRSCHEMA.description, name="eventSeries__description", curie=CRSCHEMA.curie('description'),
                   model_uri=CRSCHEMA.eventSeries__description, domain=None, range=Optional[str])

slots.eventSeries__title = Slot(uri=CRSCHEMA.title, name="eventSeries__title", curie=CRSCHEMA.curie('title'),
                   model_uri=CRSCHEMA.eventSeries__title, domain=None, range=Optional[str])

slots.eventSeries__wikidataid = Slot(uri=CRSCHEMA.wikidataid, name="eventSeries__wikidataid", curie=CRSCHEMA.curie('wikidataid'),
                   model_uri=CRSCHEMA.eventSeries__wikidataid, domain=None, range=Optional[str])

slots.eventSeries__homepage = Slot(uri=CRSCHEMA.homepage, name="eventSeries__homepage", curie=CRSCHEMA.curie('homepage'),
                   model_uri=CRSCHEMA.eventSeries__homepage, domain=None, range=Optional[str])

slots.event__acronym = Slot(uri=CRSCHEMA.acronym, name="event__acronym", curie=CRSCHEMA.curie('acronym'),
                   model_uri=CRSCHEMA.event__acronym, domain=None, range=Optional[str])

slots.event__description = Slot(uri=CRSCHEMA.description, name="event__description", curie=CRSCHEMA.curie('description'),
                   model_uri=CRSCHEMA.event__description, domain=None, range=Optional[str])

slots.event__title = Slot(uri=CRSCHEMA.title, name="event__title", curie=CRSCHEMA.curie('title'),
                   model_uri=CRSCHEMA.event__title, domain=None, range=Optional[str])

slots.event__homepage = Slot(uri=CRSCHEMA.homepage, name="event__homepage", curie=CRSCHEMA.curie('homepage'),
                   model_uri=CRSCHEMA.event__homepage, domain=None, range=Optional[str])

slots.event__wikidataid = Slot(uri=CRSCHEMA.wikidataid, name="event__wikidataid", curie=CRSCHEMA.curie('wikidataid'),
                   model_uri=CRSCHEMA.event__wikidataid, domain=None, range=Optional[str])

slots.event__series = Slot(uri=CRSCHEMA.series, name="event__series", curie=CRSCHEMA.curie('series'),
                   model_uri=CRSCHEMA.event__series, domain=None, range=Optional[Union[dict, EventSeries]])

slots.event__city = Slot(uri=CRSCHEMA.city, name="event__city", curie=CRSCHEMA.curie('city'),
                   model_uri=CRSCHEMA.event__city, domain=None, range=Optional[Union[dict, City]])

slots.event__country = Slot(uri=CRSCHEMA.country, name="event__country", curie=CRSCHEMA.curie('country'),
                   model_uri=CRSCHEMA.event__country, domain=None, range=Optional[Union[dict, Country]])

slots.event__region = Slot(uri=CRSCHEMA.region, name="event__region", curie=CRSCHEMA.curie('region'),
                   model_uri=CRSCHEMA.event__region, domain=None, range=Optional[Union[dict, Region]])

slots.city__name = Slot(uri=CRSCHEMA.name, name="city__name", curie=CRSCHEMA.curie('name'),
                   model_uri=CRSCHEMA.city__name, domain=None, range=Optional[str])

slots.city__wikidataid = Slot(uri=CRSCHEMA.wikidataid, name="city__wikidataid", curie=CRSCHEMA.curie('wikidataid'),
                   model_uri=CRSCHEMA.city__wikidataid, domain=None, range=Optional[str])

slots.country__name = Slot(uri=CRSCHEMA.name, name="country__name", curie=CRSCHEMA.curie('name'),
                   model_uri=CRSCHEMA.country__name, domain=None, range=Optional[str])

slots.country__wikidataid = Slot(uri=CRSCHEMA.wikidataid, name="country__wikidataid", curie=CRSCHEMA.curie('wikidataid'),
                   model_uri=CRSCHEMA.country__wikidataid, domain=None, range=Optional[str])

slots.region__name = Slot(uri=CRSCHEMA.name, name="region__name", curie=CRSCHEMA.curie('name'),
                   model_uri=CRSCHEMA.region__name, domain=None, range=Optional[str])

slots.region__wikidataid = Slot(uri=CRSCHEMA.wikidataid, name="region__wikidataid", curie=CRSCHEMA.curie('wikidataid'),
                   model_uri=CRSCHEMA.region__wikidataid, domain=None, range=Optional[str])

slots.proceedings__wikidataid = Slot(uri=CRSCHEMA.wikidataid, name="proceedings__wikidataid", curie=CRSCHEMA.curie('wikidataid'),
                   model_uri=CRSCHEMA.proceedings__wikidataid, domain=None, range=Optional[str])

slots.proceedings__title = Slot(uri=CRSCHEMA.title, name="proceedings__title", curie=CRSCHEMA.curie('title'),
                   model_uri=CRSCHEMA.proceedings__title, domain=None, range=Optional[str])

slots.scholar__wikiDataId = Slot(uri=CRSCHEMA.wikiDataId, name="scholar__wikiDataId", curie=CRSCHEMA.curie('wikiDataId'),
                   model_uri=CRSCHEMA.scholar__wikiDataId, domain=None, range=Optional[str])

slots.scholar__name = Slot(uri=CRSCHEMA.name, name="scholar__name", curie=CRSCHEMA.curie('name'),
                   model_uri=CRSCHEMA.scholar__name, domain=None, range=Optional[str])

slots.scholar__firstName = Slot(uri=CRSCHEMA.firstName, name="scholar__firstName", curie=CRSCHEMA.curie('firstName'),
                   model_uri=CRSCHEMA.scholar__firstName, domain=None, range=Optional[str])

slots.scholar__description = Slot(uri=CRSCHEMA.description, name="scholar__description", curie=CRSCHEMA.curie('description'),
                   model_uri=CRSCHEMA.scholar__description, domain=None, range=Optional[str])

slots.scholar__homepage = Slot(uri=CRSCHEMA.homepage, name="scholar__homepage", curie=CRSCHEMA.curie('homepage'),
                   model_uri=CRSCHEMA.scholar__homepage, domain=None, range=Optional[str])

slots.scholar__orcid = Slot(uri=CRSCHEMA.orcid, name="scholar__orcid", curie=CRSCHEMA.curie('orcid'),
                   model_uri=CRSCHEMA.scholar__orcid, domain=None, range=Optional[str])

slots.scholar__dblpId = Slot(uri=CRSCHEMA.dblpId, name="scholar__dblpId", curie=CRSCHEMA.curie('dblpId'),
                   model_uri=CRSCHEMA.scholar__dblpId, domain=None, range=Optional[str])

slots.scholar__googleScholarUser = Slot(uri=CRSCHEMA.googleScholarUser, name="scholar__googleScholarUser", curie=CRSCHEMA.curie('googleScholarUser'),
                   model_uri=CRSCHEMA.scholar__googleScholarUser, domain=None, range=Optional[str])

slots.scholar__linkedInId = Slot(uri=CRSCHEMA.linkedInId, name="scholar__linkedInId", curie=CRSCHEMA.curie('linkedInId'),
                   model_uri=CRSCHEMA.scholar__linkedInId, domain=None, range=Optional[str])

slots.scholar__researchGate = Slot(uri=CRSCHEMA.researchGate, name="scholar__researchGate", curie=CRSCHEMA.curie('researchGate'),
                   model_uri=CRSCHEMA.scholar__researchGate, domain=None, range=Optional[str])

slots.scholar__gndId = Slot(uri=CRSCHEMA.gndId, name="scholar__gndId", curie=CRSCHEMA.curie('gndId'),
                   model_uri=CRSCHEMA.scholar__gndId, domain=None, range=Optional[str])

slots.scholar__smartCRMId = Slot(uri=CRSCHEMA.smartCRMId, name="scholar__smartCRMId", curie=CRSCHEMA.curie('smartCRMId'),
                   model_uri=CRSCHEMA.scholar__smartCRMId, domain=None, range=Optional[str])

slots.institution__title = Slot(uri=CRSCHEMA.title, name="institution__title", curie=CRSCHEMA.curie('title'),
                   model_uri=CRSCHEMA.institution__title, domain=None, range=Optional[str])

slots.institution__wikidataid = Slot(uri=CRSCHEMA.wikidataid, name="institution__wikidataid", curie=CRSCHEMA.curie('wikidataid'),
                   model_uri=CRSCHEMA.institution__wikidataid, domain=None, range=Optional[str])

slots.institution__homepage = Slot(uri=CRSCHEMA.homepage, name="institution__homepage", curie=CRSCHEMA.curie('homepage'),
                   model_uri=CRSCHEMA.institution__homepage, domain=None, range=Optional[str])

slots.institution__smartCRMId = Slot(uri=CRSCHEMA.smartCRMId, name="institution__smartCRMId", curie=CRSCHEMA.curie('smartCRMId'),
                   model_uri=CRSCHEMA.institution__smartCRMId, domain=None, range=Optional[str])

slots.institution__scholar = Slot(uri=CRSCHEMA.scholar, name="institution__scholar", curie=CRSCHEMA.curie('scholar'),
                   model_uri=CRSCHEMA.institution__scholar, domain=None, range=Optional[Union[dict, Scholar]])

slots.paper__description = Slot(uri=CRSCHEMA.description, name="paper__description", curie=CRSCHEMA.curie('description'),
                   model_uri=CRSCHEMA.paper__description, domain=None, range=Optional[str])

slots.paper__id = Slot(uri=CRSCHEMA.id, name="paper__id", curie=CRSCHEMA.curie('id'),
                   model_uri=CRSCHEMA.paper__id, domain=None, range=Optional[str])

slots.paper__wikidataid = Slot(uri=CRSCHEMA.wikidataid, name="paper__wikidataid", curie=CRSCHEMA.curie('wikidataid'),
                   model_uri=CRSCHEMA.paper__wikidataid, domain=None, range=Optional[str])

slots.paper__title = Slot(uri=CRSCHEMA.title, name="paper__title", curie=CRSCHEMA.curie('title'),
                   model_uri=CRSCHEMA.paper__title, domain=None, range=Optional[str])

slots.paper__authors = Slot(uri=CRSCHEMA.authors, name="paper__authors", curie=CRSCHEMA.curie('authors'),
                   model_uri=CRSCHEMA.paper__authors, domain=None, range=Optional[str])

slots.paper__pdfUrl = Slot(uri=CRSCHEMA.pdfUrl, name="paper__pdfUrl", curie=CRSCHEMA.curie('pdfUrl'),
                   model_uri=CRSCHEMA.paper__pdfUrl, domain=None, range=Optional[Union[str, URI]])



'''
Created on 2023-02-19

@author: wf
'''
from argparse import ArgumentParser
from argparse import RawDescriptionHelpFormatter
import os
import sys
import traceback
from meta.version import Version
from meta.mw import SMWAccess
from meta.uml import PlantUml
from meta.sidif2linkml import SiDIF2LinkML
from meta.metamodel import Context
import webbrowser

class MetaModelCmd:
    """
    command line interface for metamodel handling
    """    
    
    def __init__(self,debug:bool=False):
        """
        constructor
        
        Args:
            debug(bool): if True switch debugging on
        """
        self.debug=debug
        self.error=None
        self.errMsg=None
        self.context=None
        
    @classmethod
    def getArgParser(cls):
        """
        get my argument parser
        """
        program_shortdesc = Version.description        
        program_license = '''%s
        
          Created by %s on %s.
          Copyright 2022-2023 Wolfgang Fahl. All rights reserved.
        
          Licensed under the Apache License 2.0
          http://www.apache.org/licenses/LICENSE-2.0
        
          Distributed on an "AS IS" basis without warranties
          or conditions of any kind, either express or implied.
        
        USAGE
        ''' % (program_shortdesc, Version.authors,str(__date__))
        parser = ArgumentParser(description=program_license, formatter_class=RawDescriptionHelpFormatter)
        parser.add_argument("--about",help="show about info [default: %(default)s]",action="store_true")
        parser.add_argument('--context', default="MetaModel",help='context to read [default: %(default)s]')
        parser.add_argument("--copyright",help="copyright message for diagrams")
        parser.add_argument("-d", "--debug", dest="debug", action="store_true", help="show debug info")
        parser.add_argument("-i", "--input", help="input file")
        parser.add_argument("-l", "--linkml", action="store_true", help="create linkml yaml file")
        parser.add_argument("-t", "--title",help="the title of a diagram")
        parser.add_argument("-u", "--uml", action="store_true", help="create plantuml diagram")
        parser.add_argument("-at","--withAt",action="store_true",help="generate with @startuml/@enduml bracket")
        parser.add_argument('--wikiId', default="wiki",help='id of the wiki to generate for [default: %(default)s]')
        return parser

    def readContext(self,args):
        """
        read the context from the given args
        
        Args:
            args(Args): command line arguments
        """
        if args.input:
            result_tuple=Context.fromSiDIF_input(args.input,debug=args.debug)
            self.context,self.error,self.errMsg=result_tuple
        elif args.wikiId and args.context:
            self.wikiId=args.wikiId
            self.smwAccess=SMWAccess(args.wikiId)
            self.mw_contexts=self.smwAccess.getMwContexts()
            self.mw_context=self.mw_contexts.get(args.context,None)
            self.context,self.error,self.errMsg=Context.fromWikiContext(self.mw_context, debug=args.debug)
         
    def hasError(self):  
        if self.error is not None:
            print(f"reading Context failed:\n{self.errMsg}\n{self.error}")
        return self.error
         
    def genUml(self,args)->str:
        """
        generate uml for the given context with the given name from the given wiki
        
        Args:
           args: the command line arguments
           
        Returns:
            str: the uml markup
        """
        self.readContext(args)  
        if not self.hasError():
            uml=PlantUml(title=args.title,copyRight=args.copyright,withAt=args.withAt)
            uml.fromDIF(self.context.dif)
            return str(uml)
        
    def genLinkML(self,args)->str:
        """
        generate linkML yaml for the given command line arguments
        
        Args:
           args: the command line arguments
           
        Returns:
            str: the uml markup
        """
        self.readContext(args)  
        if not self.hasError():
            sidif2LinkML=SiDIF2LinkML(self.context)
            yaml_text=sidif2LinkML.asYaml()
            return yaml_text

__version__ = Version.version
__date__ = Version.date
__updated__ = Version.updated 


def main(argv=None): # IGNORE:C0111
    '''main program.'''

    if argv is None:
        argv=sys.argv[1:]
    
    program_name = os.path.basename(__file__)
    program_version =f"v{__version__}" 
    program_build_date = str(__updated__)
    program_version_message = f'{program_name} ({program_version},{program_build_date})'
   
    try:
        parser=MetaModelCmd.getArgParser()
        # Setup argument parser
        if len(argv) < 1:
            parser.print_usage()
            sys.exit(1)
            
        args = parser.parse_args(argv)
        mm_cmd=MetaModelCmd(debug=args.debug)
        if args.about:
            print(program_version_message)
            print(f"see {Version.doc_url}")
            webbrowser.open(Version.doc_url)
        if args.uml:
            uml=mm_cmd.genUml(args)
            print(uml)
        elif args.linkml:
            mm_cmd=MetaModelCmd(debug=args.debug)
            linkml_yaml=mm_cmd.genLinkML(args)
            print (linkml_yaml)
    
    except KeyboardInterrupt:
        ### handle keyboard interrupt ###
        return 1
    except Exception as e:
        if DEBUG:
            raise(e)
        indent = len(program_name) * " "
        sys.stderr.write(program_name + ": " + repr(e) + "\n")
        sys.stderr.write(indent + "  for help use --help")
        if args.debug:
            print(traceback.format_exc())
        return 2       

DEBUG = 1
if __name__ == "__main__":
    if DEBUG:
        sys.argv.append("-d")
    sys.exit(main())
    



'''
Created on 2023-02-20

@author: wf
'''
from meta.metamodel_cmd import MetaModelCmd
from meta.sidif2linkml import SiDIF2LinkML
from tests.basemwtest import BaseMediawikiTest

class TestLinkML(BaseMediawikiTest):
    """
    test linkml handling
    
    https://linkml.io/
    """
    
    def setUp(self, debug=False, profile=True):
        """
        setUp
        """
        BaseMediawikiTest.setUp(self, debug=debug, profile=profile)
        for wikiId in ["wiki","cr"]:
            self.getWikiUser(wikiId, save=True)
    
    def testSiDIF2LinkML(self):
        """
        test conversion from SiDIF to LinkML
        """
        debug=self.debug
        debug=True
        parser=MetaModelCmd.getArgParser()
        mm_cmd=MetaModelCmd(debug=debug)
        for wikiId,context_name in [
            ("wiki","MetaModel"),
            ("cr","CrSchema")
        ]:
            args=parser.parse_args(["--wikiId",wikiId,"--context",context_name])
            mm_cmd.readContext(args)
            self.assertTrue(not mm_cmd.hasError())
            s2l=SiDIF2LinkML(mm_cmd.context)
            yaml_text=s2l.asYaml()
            if debug:
                print(yaml_text)


'''
Created on 2023-02-20

@author: wf
'''
from meta.metamodel import Context
from linkml_runtime.utils.schemaview import SchemaView
from linkml_runtime.linkml_model import SchemaDefinition, ClassDefinition, SlotDefinition
from linkml.generators.linkmlgen import LinkmlGenerator
from linkml_runtime.linkml_model import Prefix
from linkml.utils.schema_builder import SchemaBuilder


class SiDIF2LinkML:
    """
    converter between SiDIF and LINKML
    """
    
    def __init__(self,context:Context):
        self.context=context
        
    def asYaml(self,common_property_slots:bool=True,delim="_")->str:
        """
        convert my context
        
        Args:
            common_property_slots(bool): if True reuse slots 
            
        Returns:
            str: the yaml markup
        
        """
        context=self.context
        sb=SchemaBuilder(id=context.name,name=context.name)
        # https://linkml.io/linkml-model/docs/SchemaDefinition/
        sb.add_defaults()
        sd=sb.schema
        sv=SchemaView(sd)
        if hasattr(context,"copyright"):
            copyright_str=f" copyright {context.copyright}"
        else:
            copyright_str=""
        if hasattr(context,"master"):
            master=context.master
        else:
            master="http://example.com"
        uri=f"{master}/{context.name}"
        for topic in self.context.topics.values():
            cd=ClassDefinition(name=topic.name)
            cd.description=topic.documentation
            sv.add_class(cd)
            for prop in topic.properties.values():
                slot=None
                if common_property_slots:
                    qname=prop.name
                    if prop.name in sd.slots:
                        slot=sd.slots[prop.name]
                        slot.description+=","+prop.documentation
                else:
                    qname=f"{topic.name}{delim}{prop.name}"
                if slot is None:
                    slot=SlotDefinition(name=qname)
                    if hasattr(prop,"documentation"):
                        slot.description=prop.documentation  
                    typesMap={
                        "Boolean": "boolean",
                        "Code": "string", # @TODO will need special type
                        "Date": "date",
                        "Text": "string",
                        "URL": "uri",
                        "Page": "string", # @TODO will need own type
                        "Number": "double", # @TODO will need special type handling
                    }
                    if prop.isLink:
                        # 1:1 and 1:n handling
                        slot.range=prop.topicLink.source
                    else:
                        if prop.type in typesMap:
                            slot.range=typesMap[prop.type]
                        else:
                            slot.range="string"       
                    sv.add_slot(slot)
                cd.attributes[qname]=slot
                pass
            
            pass
      
        lml_gen=LinkmlGenerator(schema=sd,format='yaml')
        yaml_text=lml_gen.serialize()
        return yaml_text